@@ -1,5 +1,5 @@
 import numpy as np
-from typing import Callable, Tuple, Union
+from typing import Callable, Tuple, Union, Sequence
 
 
 def attributes_check(metric):
@@ -46,22 +46,6 @@ def assert_model_predictions_correct(
         return False
 
 
-def set_warn(call):
-    # TODO. Implement warning logic of decorator if text_warning is an attribute in class.
-    def call_fn(*args):
-        return call_fn
-
-    return call
-    # attr = call.__dict__
-    # print(dir(call))
-    # attr = {}
-    # if "text_warning" in attr:
-    #    call.print_warning(text=attr["text_warning"])
-    # else:
-    #    print("Do nothing.")
-    #    pass
-
-
 def assert_features_in_step(
     features_in_step: int, input_shape: Tuple[int, ...]
 ) -> None:
@@ -73,14 +57,6 @@ def assert_features_in_step(
     )
 
 
-def assert_max_steps(max_steps_per_input: int, input_shape: Tuple[int, ...]) -> None:
-    """Assert that max steps per inputs is compatible with the image size."""
-    assert np.prod(input_shape) % max_steps_per_input == 0, (
-        "Set 'max_steps_per_input' so that the modulo remainder "
-        "returns zero given the product of the input shape."
-    )
-
-
 def assert_patch_size(patch_size: int, shape: Tuple[int, ...]) -> None:
     """Assert that patch size is compatible with given shape."""
     if isinstance(patch_size, int):
@@ -122,7 +98,7 @@ def assert_nr_segments(nr_segments: int) -> None:
 
 
 def assert_perturbation_caused_change(x: np.ndarray, x_perturbed: np.ndarray) -> None:
-    """Assert that perturbation applied to input caused change so that input and perturbed input is not the smae."""
+    """Assert that perturbation applied to input caused change so that input and perturbed input is not the same."""
     assert (x.flatten() != x_perturbed.flatten()).any(), (
         "The settings for perturbing input e.g., 'perturb_func' "
         "didn't cause change in input. "
@@ -157,11 +133,31 @@ def assert_attributions(x_batch: np.array, a_batch: np.array) -> None:
         "include the same number of samples."
         "{} != {}".format(np.shape(x_batch)[0], np.shape(a_batch)[0])
     )
-
-    assert np.shape(x_batch)[2:] == np.shape(a_batch)[2:], (
-        "The inputs 'x_batch' and attributions 'a_batch' "
-        "should share the same dimensions."
-        "{} != {}".format(np.shape(x_batch)[2:], np.shape(a_batch)[2:])
+    assert np.ndim(x_batch) == np.ndim(a_batch), (
+        "The inputs 'x_batch' and attributions 'a_batch' should "
+        "have the same number of dimensions."
+        "{} != {}".format(np.ndim(x_batch), np.ndim(a_batch))
+    )
+    a_shape = [s for s in np.shape(a_batch)[1:] if s != 1]
+    x_shape = [s for s in np.shape(x_batch)[1:]]
+    assert a_shape[0] == x_shape[0] or a_shape[-1] == x_shape[-1], (
+        "The dimensions of attribution and input per sample should correspond in either "
+        "the first or last dimensions, but got shapes "
+        "{} and {}".format(a_shape, x_shape)
+    )
+    assert all([a in x_shape for a in a_shape]), (
+        "All attribution dimensions should be included in the input dimensions, "
+        "but got shapes {} and {}".format(a_shape, x_shape)
+    )
+    assert all(
+        [
+            x_shape.index(a) > x_shape.index(a_shape[i])
+            for a in a_shape
+            for i in range(a_shape.index(a))
+        ]
+    ), (
+        "The dimensions of the attribution must correspond to dimensions of the input in the same order, "
+        "but got shapes {} and {}".format(a_shape, x_shape)
     )
     assert not np.all((a_batch == 0)), (
         "The elements in the attribution vector are all equal to zero, "
@@ -232,3 +228,23 @@ def assert_value_smaller_than_input_size(x: np.ndarray, value: int, value_name:
             f"'{value_name}' must be smaller than input size."
             f" [{value} >= {np.prod(x.shape[2:])}]"
         )
+
+
+# TODO: Change for batching update, since currently single images are expected
+def assert_indexed_axes(arr: np.array, indexed_axes: Sequence[int]):
+    """
+    Checks that indexed_axes fits arr
+    """
+    assert len(indexed_axes) <= arr.ndim
+    assert len(indexed_axes) == len(np.arange(indexed_axes[0], indexed_axes[-1] + 1))
+    assert all(
+        [
+            a == b
+            for a, b in list(
+                zip(indexed_axes, np.arange(indexed_axes[0], indexed_axes[-1] + 1))
+            )
+        ]
+    ), "Make sure indexed_axes contains consecutive axes."
+    assert (
+        0 in indexed_axes or arr.ndim - 1 in indexed_axes
+    ), "Make sure indexed_axes contains either the first or last axis of arr."
