@@ -3,10 +3,13 @@ from typing import Dict, Optional, Union
 
 import numpy as np
 import scipy
-import random
 from importlib import util
 import cv2
 import warnings
+from .utils import *
+from .normalise_func import *
+from ..helpers import __EXTRAS__
+from ..helpers import constants
 
 if util.find_spec("torch"):
     import torch
@@ -99,6 +102,8 @@ def generate_tf_explanation(
     """
     method = kwargs.get("method", "Gradient").lower()
     inputs = inputs.reshape(-1, *model.input_shape[1:])
+    if not isinstance(targets, np.ndarray):
+        targets = np.array([targets])
 
     channel_first = kwargs.get("channel_first", infer_channel_first(inputs))
     inputs = make_channel_last(inputs, channel_first)
@@ -177,7 +182,7 @@ def generate_tf_explanation(
     elif method == "GradCam".lower():
         if "gc_layer" not in kwargs:
             raise ValueError(
-                "Specify convolutional layer name as 'gc_layer' to run GradCam."
+                "Specify a convolutional layer name as 'gc_layer' to run GradCam."
             )
 
         explainer = tf_explain.core.grad_cam.GradCAM()
@@ -199,18 +204,20 @@ def generate_tf_explanation(
 
     else:
         raise KeyError(
-            "Specify a XAI method that already has been implemented {}."
-        ).__format__("XAI_METHODS")
+            f"Specify a XAI method that already has been implemented {constants.AVAILABLE_XAI_METHODS}."
+        )
 
     if (
         not kwargs.get("normalise", True)
         or not kwargs.get("abs", True)
         or not kwargs.get("pos_only", True)
         or kwargs.get("neg_only", False)
+        or kwargs.get("reduce_axes", None) is not None
     ):
         raise KeyError(
             "Only normalized absolute explanations are currently supported for TensorFlow models (tf-explain). "
-            "Set normalise=true, abs=true, pos_only=true, neg_only=false."
+            "Set normalise=true, abs=true, pos_only=true, neg_only=false. reduce_axes parameter is not available; "
+            "explanations are always reduced over channel axis."
         )
 
     return explanation
@@ -224,7 +231,9 @@ def generate_captum_explanation(
     **kwargs,
 ) -> np.ndarray:
     """Generate explanation for a torch model with captum."""
+
     method = kwargs.get("method", "Gradient").lower()
+
     # Set model in evaluate mode.
     model.to(device)
     model.eval()
@@ -235,14 +244,15 @@ def generate_captum_explanation(
     if not isinstance(targets, torch.Tensor):
         targets = torch.as_tensor(targets).to(device)
 
-    """ TODO: why is this needed?
-    inputs = inputs.reshape(
-        -1,
-        kwargs.get("nr_channels", 3),
-        kwargs.get("img_size", 224),
-        kwargs.get("img_size", 224),
+    assert 0 not in kwargs.get(
+        "reduce_axes", [1]
+    ), "Reduction over batch_axis is not available, please do not include axis 0 in 'reduce_axes' kwargs."
+    assert len(kwargs.get("reduce_axes", [1])) <= inputs.ndim - 1, (
+        "Cannot reduce attributions over more axes than each sample has dimensions, but got "
+        "{} and  {}.".format(len(kwargs.get("reduce_axes", [1])), inputs.ndim - 1)
     )
-    """
+
+    reduce_axes = {"axis": tuple(kwargs.get("reduce_axes", [1])), "keepdims": True}
 
     explanation: torch.Tensor = torch.zeros_like(inputs)
 
@@ -254,7 +264,7 @@ def generate_captum_explanation(
                 target=targets,
                 baselines=kwargs.get("baseline", torch.zeros_like(inputs)),
             )
-            .sum(axis=1)
+            .sum(**reduce_axes)
         )
 
     elif method == "IntegratedGradients".lower():
@@ -267,26 +277,28 @@ def generate_captum_explanation(
                 n_steps=10,
                 method="riemann_trapezoid",
             )
-            .sum(axis=1)
+            .sum(**reduce_axes)
         )
 
     elif method == "InputXGradient".lower():
         explanation = (
-            InputXGradient(model).attribute(inputs=inputs, target=targets).sum(axis=1)
+            InputXGradient(model)
+            .attribute(inputs=inputs, target=targets)
+            .sum(**reduce_axes)
         )
 
     elif method == "Saliency".lower():
         explanation = (
             Saliency(model)
             .attribute(inputs=inputs, target=targets, abs=True)
-            .sum(axis=1)
+            .sum(**reduce_axes)
         )
 
     elif method == "Gradient".lower():
         explanation = (
             Saliency(model)
             .attribute(inputs=inputs, target=targets, abs=False)
-            .sum(axis=1)
+            .sum(**reduce_axes)
         )
 
     elif method == "Occlusion".lower():
@@ -298,12 +310,14 @@ def generate_captum_explanation(
                 target=targets,
                 sliding_window_shapes=window_shape,
             )
-            .sum(axis=1)
+            .sum(**reduce_axes)
         )
 
     elif method == "FeatureAblation".lower():
         explanation = (
-            FeatureAblation(model).attribute(inputs=inputs, target=targets).sum(axis=1)
+            FeatureAblation(model)
+            .attribute(inputs=inputs, target=targets)
+            .sum(**reduce_axes)
         )
 
     elif method == "GradCam".lower():
@@ -318,47 +332,43 @@ def generate_captum_explanation(
         explanation = (
             LayerGradCam(model, layer=kwargs["gc_layer"])
             .attribute(inputs=inputs, target=targets)
-            .sum(axis=1)
+            .sum(**reduce_axes)
         )
-        """ TODO: why is this needed?
-        explanation = torch.Tensor(
-            cv2.resize(
-                explanation.cpu().data.numpy(),
-                dsize=(kwargs.get("img_size", 224), kwargs.get("img_size", 224)),
-            )
-        )
-        """
 
     elif method == "Control Var. Sobel Filter".lower():
-        explanation = torch.zeros(size=(inputs.shape[0], *inputs.shape[2:]))
+        explanation = torch.zeros(size=inputs.shape)
 
         for i in range(len(explanation)):
             explanation[i] = torch.Tensor(
-                np.clip(scipy.ndimage.sobel(inputs[i].cpu().numpy()), 0, 1).mean(axis=0)
-                # TODO: why is this needed?
-                # .reshape(kwargs.get("img_size", 224), kwargs.get("img_size", 224))
+                np.clip(scipy.ndimage.sobel(inputs[i].cpu().numpy()), 0, 1)
             )
+        explanation = explanation.mean(**reduce_axes)
+
+    elif method == "Control Var. Random Uniform".lower():
+        explanation = torch.rand(size=(inputs.shape[0], *inputs.shape[2:]))
 
     elif method == "Control Var. Constant".lower():
         assert (
             "constant_value" in kwargs
         ), "Specify a 'constant_value' e.g., 0.0 or 'black' for pixel replacement."
 
-        explanation = torch.zeros(size=(inputs.shape[0], *inputs.shape[2:]))
+        explanation = torch.zeros(size=inputs.shape)
 
         # Update the tensor with values per input x.
         for i in range(explanation.shape[0]):
             constant_value = get_baseline_value(
-                choice=kwargs["constant_value"], arr=inputs[i]
-            )
+                value=kwargs["constant_value"], arr=inputs[i], return_shape=(1,)
+            )[0]
             explanation[i] = torch.Tensor().new_full(
                 size=explanation[0].shape, fill_value=constant_value
             )
 
+        explanation = explanation.mean(**reduce_axes)
+
     else:
         raise KeyError(
-            "Specify a XAI method that already has been implemented {}."
-        ).__format__("XAI_METHODS")
+            f"Specify a XAI method that already has been implemented {constants.AVAILABLE_XAI_METHODS}."
+        )
 
     if isinstance(explanation, torch.Tensor):
         if explanation.requires_grad:
@@ -389,8 +399,18 @@ def generate_zennit_explanation(
     **kwargs,
 ) -> np.ndarray:
     """Generate explanation for a torch model with zennit."""
-    # Get zennit composite, canonizer, attributor
-    # Handle canonizer kwarg
+
+    assert 0 not in kwargs.get(
+        "reduce_axes", [1]
+    ), "Reduction over batch_axis is not available, please do not include axis 0 in 'reduce_axes' kwarg."
+    assert len(kwargs.get("reduce_axes", [1])) <= inputs.ndim - 1, (
+        "Cannot reduce attributions over more axes than each sample has dimensions, but got "
+        "{} and  {}.".format(len(kwargs.get("reduce_axes", [1])), inputs.ndim - 1)
+    )
+
+    reduce_axes = {"axis": tuple(kwargs.get("reduce_axes", [1])), "keepdims": True}
+
+    # Get zennit composite, canonizer, attributor and handle canonizer kwargs.
     canonizer = kwargs.get("canonizer", None)
     if not canonizer == None and not issubclass(canonizer, zcanon.Canonizer):
         raise ValueError(
@@ -398,8 +418,7 @@ def generate_zennit_explanation(
             "Please provide None or an instance of zennit.canonizers.Canonizer"
         )
 
-    # Handle attributor kwarg
-    # TODO: we could create a str --> attributor mapping, but I like this better
+    # Handle attributor kwargs.
     attributor = kwargs.get("attributor", zattr.Gradient)
     if not issubclass(attributor, zattr.Attributor):
         raise ValueError(
@@ -407,7 +426,7 @@ def generate_zennit_explanation(
             "Please provide a subclass of zennit.attributon.Attributor"
         )
 
-    # Handle composite kwarg
+    # Handle attributor kwargs.
     composite = kwargs.get("composite", None)
     if not composite == None and isinstance(composite, str):
         if composite not in zcomp.COMPOSITES.keys():
@@ -426,6 +445,7 @@ def generate_zennit_explanation(
                 composite, zcomp.COMPOSITES.keys()
             )
         )
+
     # Set model in evaluate mode.
     model.eval()
 
@@ -435,38 +455,41 @@ def generate_zennit_explanation(
     if not isinstance(targets, torch.Tensor):
         targets = torch.as_tensor(targets).to(device)
 
-    """ TODO: why is this needed?
-    inputs = inputs.reshape(
-        -1,
-        kwargs.get("nr_channels", 3),
-        kwargs.get("img_size", 224),
-        kwargs.get("img_size", 224),
-    )
-    """
+    canonizer_kwargs = kwargs.get("canonizer_kwargs", {})
+    composite_kwargs = kwargs.get("composite_kwargs", {})
+    attributor_kwargs = kwargs.get("attributor_kwargs", {})
 
-    # Initialize canonizer, composite, and attributor
+    # Initialize canonizer, composite, and attributor.
     if canonizer is not None:
-        canonizers = [canonizer()]
+        canonizers = [canonizer(**canonizer_kwargs)]
     else:
         canonizers = []
     if composite is not None:
-        # TODO: only uses default parameters for each method for now
-        composite = composite(canonizers=canonizers)
-    # TODO: only uses default parameters for each method for now
-    attributor = attributor(model, composite)
+        composite = composite(
+            **{
+                **composite_kwargs,
+                "canonizers": canonizers,
+            }
+        )
+    attributor = attributor(
+        **{
+            **attributor_kwargs,
+            "model": model,
+            "composite": composite,
+        }
+    )
 
-    # TODO: there may be a better solution here?
     n_outputs = model(inputs).shape[1]
 
-    # Get Attributions
+    # Get the attributions.
     with attributor:
-
-        # TODO: this assumes one-hot encoded target outputs (e.g., initial relevance).
-        #  Better solution with more choices?
-        eye = torch.eye(n_outputs, device=device)
-        output_target = eye[targets]
-        output_target = output_target.reshape(-1, n_outputs)
-        _, explanation = attributor(inputs, output_target)
+        if "attr_output" in attributor_kwargs.keys():
+            _, explanation = attributor(inputs, None)
+        else:
+            eye = torch.eye(n_outputs, device=device)
+            output_target = eye[targets]
+            output_target = output_target.reshape(-1, n_outputs)
+            _, explanation = attributor(inputs, output_target)
 
     if isinstance(explanation, torch.Tensor):
         if explanation.requires_grad:
@@ -474,9 +497,8 @@ def generate_zennit_explanation(
         else:
             explanation = explanation.cpu().numpy()
 
-    # TODO: Include alternatives here?
-    # Remove channel axis
-    explanation = np.sum(explanation, axis=1)
+    # Sum over the axes.
+    explanation = np.sum(explanation, **reduce_axes)
 
     if kwargs.get("normalise", False):
         explanation = kwargs.get("normalise_func", normalise_by_negative)(explanation)
