@@ -1,4 +1,6 @@
 from abc import ABC, abstractmethod
+from typing import Optional, Tuple
+import numpy as np
 
 
 class ModelInterface(ABC):
@@ -14,8 +16,13 @@ class ModelInterface(ABC):
         raise NotImplementedError
 
     @abstractmethod
-    def shape_input(self, x):
-        """Reshape input into model expected input."""
+    def shape_input(
+        self, x: np.array, shape: Tuple[int, ...], channel_first: Optional[bool] = None
+    ):
+        """
+        Reshape input into model expected input.
+        channel_first: Explicitely state if x is formatted channel first (optional).
+        """
         raise NotImplementedError
 
     @abstractmethod
