@@ -12,7 +12,10 @@ from ..helpers import warn_func
 from ..helpers.asserts import attributes_check
 from ..helpers.model_interface import ModelInterface
 from ..helpers.normalise_func import normalise_by_negative
-from ..helpers.perturb_func import baseline_replacement_by_indices
+from ..helpers.perturb_func import (
+    baseline_replacement_by_indices,
+    baseline_replacement_by_shift,
+)
 
 
 class Completeness(Metric):
@@ -56,6 +59,7 @@ class Completeness(Metric):
             perturb_baseline (string): Indicates the type of baseline: "mean", "random", "uniform", "black" or "white",
             default="black".
             perturb_func (callable): Input perturbation function, default=baseline_replacement_by_indices.
+            softmax (boolean): Indicates wheter to use softmax probabilities or logits in model prediction.
         """
         super().__init__()
 
@@ -68,10 +72,11 @@ class Completeness(Metric):
         self.disable_warnings = self.kwargs.get("disable_warnings", False)
         self.display_progressbar = self.kwargs.get("display_progressbar", False)
         self.output_func = self.kwargs.get("output_func", lambda x: x)
-        self.perturb_baseline = self.kwargs.get("perturb_baseline", "black")
         self.perturb_func = self.kwargs.get(
             "perturb_func", baseline_replacement_by_indices
         )
+        self.perturb_baseline = self.kwargs.get("perturb_baseline", "black")
+        self.softmax = self.kwargs.get("softmax", False)
         self.last_results = []
         self.all_results = []
 
@@ -141,7 +146,7 @@ class Completeness(Metric):
 
             # Initialise the metric and evaluate explanations by calling the metric instance.
             >> metric = Completeness(abs=True, normalise=False)
-            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{}}
+            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{})
         """
         # Reshape input batch to channel first order:
         if "channel_first" in kwargs and isinstance(kwargs["channel_first"], bool):
@@ -150,7 +155,7 @@ class Completeness(Metric):
             channel_first = utils.infer_channel_first(x_batch)
         x_batch_s = utils.make_channel_first(x_batch, channel_first)
 
-        # Wrap the model into an interface
+        # Wrap the model into an interface.
         if model:
             model = utils.get_wrapped_model(model, channel_first)
 
@@ -178,12 +183,14 @@ class Completeness(Metric):
                 targets=y_batch,
                 **self.kwargs,
             )
+
+        # Expand attributions to input dimensionality.
         a_batch = utils.expand_attribution_channel(a_batch, x_batch_s)
 
         # Asserts.
         asserts.assert_attributions(a_batch=a_batch, x_batch=x_batch_s)
 
-        # use tqdm progressbar if not disabled
+        # Use tqdm progressbar if not disabled.
         if not self.display_progressbar:
             iterator = zip(x_batch_s, y_batch, a_batch)
         else:
@@ -191,29 +198,28 @@ class Completeness(Metric):
 
         for x, y, a in iterator:
 
+            if self.normalise:
+                a = self.normalise_func(a)
+
             if self.abs:
                 a = np.abs(a)
 
-            if self.normalise:
-                a = self.normalise_func(a)
+            print(self.kwargs)
 
             x_baseline = self.perturb_func(
                 arr=x,
                 indices=np.arange(0, x.size),
+                indexed_axes=np.arange(0, x.ndim),
                 **self.kwargs,
             )
 
             # Predict on input.
             x_input = model.shape_input(x, x.shape, channel_first=True)
-            y_pred = float(
-                model.predict(x_input, softmax_act=False, **self.kwargs)[:, y]
-            )
+            y_pred = float(model.predict(x_input, **self.kwargs)[:, y])
 
             # Predict on baseline.
             x_input = model.shape_input(x_baseline, x.shape, channel_first=True)
-            y_pred_baseline = float(
-                model.predict(x_input, softmax_act=False, **self.kwargs)[:, y]
-            )
+            y_pred_baseline = float(model.predict(x_input, **self.kwargs)[:, y])
 
             if np.sum(a) == self.output_func(y_pred - y_pred_baseline):
                 self.last_results.append(True)
@@ -261,6 +267,7 @@ class NonSensitivity(Metric):
             perturb_baseline (string): Indicates the type of baseline: "mean", "random", "uniform", "black" or "white",
             default="black".
             perturb_func (callable): Input perturbation function, default=baseline_replacement_by_indices.
+            softmax (boolean): Indicates wheter to use softmax probabilities or logits in model prediction.
         """
         super().__init__()
 
@@ -279,7 +286,7 @@ class NonSensitivity(Metric):
         )
         self.perturb_baseline = self.kwargs.get("perturb_baseline", "black")
         self.features_in_step = self.kwargs.get("features_in_step", 1)
-        self.max_steps_per_input = self.kwargs.get("max_steps_per_input", None)
+        self.softmax = self.kwargs.get("softmax", True)
         self.last_results = []
         self.all_results = []
 
@@ -350,7 +357,7 @@ class NonSensitivity(Metric):
 
             # Initialise the metric and evaluate explanations by calling the metric instance.
             >> metric = NonSensitivity(abs=True, normalise=False)
-            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{}}
+            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{})
         """
         # Reshape input batch to channel first order:
         if "channel_first" in kwargs and isinstance(kwargs["channel_first"], bool):
@@ -385,7 +392,10 @@ class NonSensitivity(Metric):
                 targets=y_batch,
                 **self.kwargs,
             )
+
+        # Expand attributions to input dimensionality and infer input dimensions covered by the attributions.
         a_batch = utils.expand_attribution_channel(a_batch, x_batch_s)
+        a_axes = utils.infer_attribution_axes(a_batch, x_batch_s)
 
         # Asserts.
         asserts.assert_attributions(a_batch=a_batch, x_batch=x_batch_s)
@@ -393,17 +403,8 @@ class NonSensitivity(Metric):
             features_in_step=self.features_in_step,
             input_shape=x_batch_s.shape[2:],
         )
-        if self.max_steps_per_input is not None:
-            asserts.assert_max_steps(
-                max_steps_per_input=self.max_steps_per_input,
-                input_shape=x_batch_s.shape[2:],
-            )
-            self.features_in_step = utils.get_features_in_step(
-                max_steps_per_input=self.max_steps_per_input,
-                input_shape=x_batch_s.shape[2:],
-            )
 
-        # use tqdm progressbar if not disabled
+        # Use tqdm progressbar if not disabled.
         if not self.display_progressbar:
             iterator = zip(x_batch_s, y_batch, a_batch)
         else:
@@ -413,12 +414,12 @@ class NonSensitivity(Metric):
 
             a = a.flatten()
 
-            if self.abs:
-                a = np.abs(a)
-
             if self.normalise:
                 a = self.normalise_func(a)
 
+            if self.abs:
+                a = np.abs(a)
+
             non_features = set(list(np.argwhere(a).flatten() < self.eps))
 
             vars = []
@@ -436,6 +437,7 @@ class NonSensitivity(Metric):
                     x_perturbed = self.perturb_func(
                         arr=x,
                         indices=a_ix,
+                        indexed_axes=a_axes,
                         **self.kwargs,
                     )
 
@@ -444,7 +446,7 @@ class NonSensitivity(Metric):
                         x_perturbed, x.shape, channel_first=True
                     )
                     y_pred_perturbed = float(
-                        model.predict(x_input, softmax_act=True, **self.kwargs)[:, y]
+                        model.predict(x_input, **self.kwargs)[:, y]
                     )
                     preds.append(y_pred_perturbed)
 
@@ -500,10 +502,10 @@ class InputInvariance(Metric):
         self.default_plot_func = Callable
         self.disable_warnings = self.kwargs.get("disable_warnings", False)
         self.display_progressbar = self.kwargs.get("display_progressbar", False)
-        self.input_shift = self.kwargs.get("input_shift", -1)
         self.perturb_func = self.kwargs.get(
-            "perturb_func", baseline_replacement_by_indices
+            "perturb_func", baseline_replacement_by_shift
         )
+        self.input_shift = self.kwargs.get("input_shift", -1)
         self.last_results = []
         self.all_results = []
 
@@ -570,7 +572,7 @@ class InputInvariance(Metric):
 
             # Initialise the metric and evaluate explanations by calling the metric instance.
             >> metric = InputInvariance(abs=True, normalise=False)
-            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{}}
+            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{})
         """
         # Reshape input batch to channel first order:
         if "channel_first" in kwargs and isinstance(kwargs["channel_first"], bool):
@@ -579,7 +581,7 @@ class InputInvariance(Metric):
             channel_first = utils.infer_channel_first(x_batch)
         x_batch_s = utils.make_channel_first(x_batch, channel_first)
 
-        # Wrap the model into an interface
+        # Wrap the model into an interface.
         if model:
             model = utils.get_wrapped_model(model, channel_first)
 
@@ -605,12 +607,14 @@ class InputInvariance(Metric):
                 targets=y_batch,
                 **self.kwargs,
             )
+
+        # Expand attributions to input dimensionality.
         a_batch = utils.expand_attribution_channel(a_batch, x_batch_s)
 
         # Asserts.
         asserts.assert_attributions(a_batch=a_batch, x_batch=x_batch)
 
-        # use tqdm progressbar if not disabled
+        # Use tqdm progressbar if not disabled.
         if not self.display_progressbar:
             iterator = zip(x_batch_s, y_batch, a_batch)
         else:
@@ -618,15 +622,16 @@ class InputInvariance(Metric):
 
         for x, y, a in iterator:
 
-            if self.abs:
-                warn_func.warn_absolutes_skipped()
-
             if self.normalise:
                 warn_func.warn_normalisation_skipped()
 
+            if self.abs:
+                warn_func.warn_absolutes_skipped()
+
             x_shifted = self.perturb_func(
                 arr=x,
                 indices=np.arange(0, x.size),
+                indexed_axes=np.arange(0, x.ndim),
                 **self.kwargs,
             )
             x_shifted = model.shape_input(x_shifted, x.shape, channel_first=True)
