@@ -128,16 +128,17 @@ class PointingGame(Metric):
 
             # Initialise the metric and evaluate explanations by calling the metric instance.
             >> metric = PointingGame(abs=True, normalise=False)
-            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{}}
+            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{})
         """
-        # Reshape input batch to channel first order:
+
+        # Reshape input batch to channel first order.
         if "channel_first" in kwargs and isinstance(kwargs["channel_first"], bool):
             channel_first = kwargs.get("channel_first")
         else:
             channel_first = utils.infer_channel_first(x_batch)
         x_batch_s = utils.make_channel_first(x_batch, channel_first)
 
-        # Wrap the model into an interface
+        # Wrap the model into an interface.
         if model:
             model = utils.get_wrapped_model(model, channel_first)
 
@@ -165,13 +166,15 @@ class PointingGame(Metric):
                 targets=y_batch,
                 **self.kwargs,
             )
+
+        # Expand attributions to input dimensionality.
         a_batch = utils.expand_attribution_channel(a_batch, x_batch_s)
 
         # Asserts.
         asserts.assert_attributions(x_batch=x_batch_s, a_batch=a_batch)
         asserts.assert_segmentations(x_batch=x_batch_s, s_batch=s_batch)
 
-        # use tqdm progressbar if not disabled
+        # Use tqdm progressbar if not disabled.
         if not self.display_progressbar:
             iterator = enumerate(zip(x_batch_s, y_batch, a_batch, s_batch))
         else:
@@ -186,12 +189,12 @@ class PointingGame(Metric):
             a = a.flatten()
             s = s.squeeze().flatten().astype(bool)
 
-            if self.abs:
-                a = np.abs(a)
-
             if self.normalise:
                 a = self.normalise_func(a)
 
+            if self.abs:
+                a = np.abs(a)
+
             # Find index of max value.
             max_index = np.where(a == np.max(a))[0]
 
@@ -332,16 +335,17 @@ class AttributionLocalisation(Metric):
 
             # Initialise the metric and evaluate explanations by calling the metric instance.
             >> metric = AttributionLocalisation(abs=True, normalise=False)
-            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{}}
+            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{})
         """
-        # Reshape input batch to channel first order:
+
+        # Reshape input batch to channel first order.
         if "channel_first" in kwargs and isinstance(kwargs["channel_first"], bool):
             channel_first = kwargs.get("channel_first")
         else:
             channel_first = utils.infer_channel_first(x_batch)
         x_batch_s = utils.make_channel_first(x_batch, channel_first)
 
-        # Wrap the model into an interface
+        # Wrap the model into an interface.
         if model:
             model = utils.get_wrapped_model(model, channel_first)
 
@@ -369,13 +373,15 @@ class AttributionLocalisation(Metric):
                 targets=y_batch,
                 **self.kwargs,
             )
+
+        # Expand attributions to input dimensionality.
         a_batch = utils.expand_attribution_channel(a_batch, x_batch_s)
 
         # Asserts.
         asserts.assert_attributions(x_batch=x_batch_s, a_batch=a_batch)
         asserts.assert_segmentations(x_batch=x_batch_s, s_batch=s_batch)
 
-        # use tqdm progressbar if not disabled
+        # Use tqdm progressbar if not disabled.
         if not self.display_progressbar:
             iterator = enumerate(zip(x_batch_s, y_batch, a_batch, s_batch))
         else:
@@ -389,15 +395,15 @@ class AttributionLocalisation(Metric):
             a = a.flatten()
             s = s.flatten().astype(bool)
 
+            if self.normalise:
+                a = self.normalise_func(a)
+
             if self.abs:
                 a = np.abs(a)
             else:
                 a = np.abs(a)
                 warn_func.warn_absolutes_applied()
 
-            if self.normalise:
-                a = self.normalise_func(a)
-
             # Asserts on attributions.
             if np.all((a < 0.0)):
                 raise ValueError("Attributions must not all be less than zero.")
@@ -552,16 +558,17 @@ class TopKIntersection(Metric):
 
             # Initialise the metric and evaluate explanations by calling the metric instance.
             >> metric = TopKIntersection(abs=True, normalise=False)
-            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{}}
+            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{})
         """
-        # Reshape input batch to channel first order:
+
+        # Reshape input batch to channel first order.
         if "channel_first" in kwargs and isinstance(kwargs["channel_first"], bool):
             channel_first = kwargs.get("channel_first")
         else:
             channel_first = utils.infer_channel_first(x_batch)
         x_batch_s = utils.make_channel_first(x_batch, channel_first)
 
-        # Wrap the model into an interface
+        # Wrap the model into an interface.
         if model:
             model = utils.get_wrapped_model(model, channel_first)
 
@@ -589,6 +596,8 @@ class TopKIntersection(Metric):
                 targets=y_batch,
                 **self.kwargs,
             )
+
+        # Expand attributions to input dimensionality.
         a_batch = utils.expand_attribution_channel(a_batch, x_batch_s)
 
         # Asserts.
@@ -598,7 +607,7 @@ class TopKIntersection(Metric):
             x=x_batch_s, value=self.k, value_name="k"
         )
 
-        # use tqdm progressbar if not disabled
+        # Use tqdm progressbar if not disabled.
         if not self.display_progressbar:
             iterator = enumerate(zip(x_batch_s, y_batch, a_batch, s_batch))
         else:
@@ -609,12 +618,12 @@ class TopKIntersection(Metric):
 
         for ix, (x, y, a, s) in iterator:
 
-            if self.abs:
-                a = np.abs(a)
-
             if self.normalise:
                 a = self.normalise_func(a)
 
+            if self.abs:
+                a = np.abs(a)
+
             top_k_binary_mask = np.zeros(a.shape)
             sorted_indices = np.argsort(a, axis=None)
             np.put_along_axis(
@@ -629,7 +638,7 @@ class TopKIntersection(Metric):
 
             # Concept influence (with size of object normalised tki score).
             if self.concept_influence:
-                tki = np.prod(x.shape[1:]) / np.sum(s) * tki
+                tki = np.prod(s.shape) / np.sum(s) * tki
 
             self.last_results.append(tki)
 
@@ -751,16 +760,17 @@ class RelevanceRankAccuracy(Metric):
 
             # Initialise the metric and evaluate explanations by calling the metric instance.
             >> metric = RelevanceRankAccuracy(abs=True, normalise=False)
-            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{}}
+            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{})
         """
-        # Reshape input batch to channel first order:
+
+        # Reshape input batch to channel first order.
         if "channel_first" in kwargs and isinstance(kwargs["channel_first"], bool):
             channel_first = kwargs.get("channel_first")
         else:
             channel_first = utils.infer_channel_first(x_batch)
         x_batch_s = utils.make_channel_first(x_batch, channel_first)
 
-        # Wrap the model into an interface
+        # Wrap the model into an interface.
         if model:
             model = utils.get_wrapped_model(model, channel_first)
 
@@ -788,13 +798,15 @@ class RelevanceRankAccuracy(Metric):
                 targets=y_batch,
                 **self.kwargs,
             )
+
+        # Expand attributions to input dimensionality.
         a_batch = utils.expand_attribution_channel(a_batch, x_batch_s)
 
         # Asserts.
         asserts.assert_attributions(x_batch=x_batch_s, a_batch=a_batch)
         asserts.assert_segmentations(x_batch=x_batch_s, s_batch=s_batch)
 
-        # use tqdm progressbar if not disabled
+        # Use tqdm progressbar if not disabled.
         if not self.display_progressbar:
             iterator = enumerate(zip(x_batch_s, y_batch, a_batch, s_batch))
         else:
@@ -808,12 +820,12 @@ class RelevanceRankAccuracy(Metric):
             a = a.flatten()
             s = np.where(s.flatten().astype(bool))[0]
 
-            if self.abs:
-                a = np.abs(a)
-
             if self.normalise:
                 a = self.normalise_func(a)
 
+            if self.abs:
+                a = np.abs(a)
+
             # Size of the ground truth mask.
             k = len(s)
 
@@ -945,16 +957,17 @@ class RelevanceMassAccuracy(Metric):
 
             # Initialise the metric and evaluate explanations by calling the metric instance.
             >> metric = RelevanceMassAccuracy(abs=True, normalise=False)
-            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{}}
+            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{})
         """
-        # Reshape input batch to channel first order:
+
+        # Reshape input batch to channel first order.
         if "channel_first" in kwargs and isinstance(kwargs["channel_first"], bool):
             channel_first = kwargs.get("channel_first")
         else:
             channel_first = utils.infer_channel_first(x_batch)
         x_batch_s = utils.make_channel_first(x_batch, channel_first)
 
-        # Wrap the model into an interface
+        # Wrap the model into an interface.
         if model:
             model = utils.get_wrapped_model(model, channel_first)
 
@@ -982,13 +995,15 @@ class RelevanceMassAccuracy(Metric):
                 targets=y_batch,
                 **self.kwargs,
             )
+
+        # Expand attributions to input dimensionality.
         a_batch = utils.expand_attribution_channel(a_batch, x_batch_s)
 
         # Asserts.
         asserts.assert_attributions(x_batch=x_batch_s, a_batch=a_batch)
         asserts.assert_segmentations(x_batch=x_batch_s, s_batch=s_batch)
 
-        # use tqdm progressbar if not disabled
+        # Use tqdm progressbar if not disabled.
         if not self.display_progressbar:
             iterator = enumerate(zip(x_batch_s, y_batch, a_batch, s_batch))
         else:
@@ -1001,12 +1016,12 @@ class RelevanceMassAccuracy(Metric):
 
             a = a.flatten()
 
-            if self.abs:
-                a = np.abs(a)
-
             if self.normalise:
                 a = self.normalise_func(a)
 
+            if self.abs:
+                a = np.abs(a)
+
             # Reshape.
             s = s.flatten().astype(bool)
 
@@ -1132,14 +1147,15 @@ class AUC(Metric):
             >> metric = AUC(abs=True, normalise=False)
             >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **params_call}
         """
-        # Reshape input batch to channel first order:
+
+        # Reshape input batch to channel first order.
         if "channel_first" in kwargs and isinstance(kwargs["channel_first"], bool):
             channel_first = kwargs.get("channel_first")
         else:
             channel_first = utils.infer_channel_first(x_batch)
         x_batch_s = utils.make_channel_first(x_batch, channel_first)
 
-        # Wrap the model into an interface
+        # Wrap the model into an interface.
         if model:
             model = utils.get_wrapped_model(model, channel_first)
 
@@ -1167,13 +1183,15 @@ class AUC(Metric):
                 targets=y_batch,
                 **self.kwargs,
             )
+
+        # Expand attributions to input dimensionality.
         a_batch = utils.expand_attribution_channel(a_batch, x_batch_s)
 
         # Asserts.
         asserts.assert_attributions(x_batch=x_batch_s, a_batch=a_batch)
         asserts.assert_segmentations(x_batch=x_batch_s, s_batch=s_batch)
 
-        # use tqdm progressbar if not disabled
+        # Use tqdm progressbar if not disabled.
         if not self.display_progressbar:
             iterator = enumerate(zip(x_batch_s, y_batch, a_batch, s_batch))
         else:
@@ -1184,12 +1202,12 @@ class AUC(Metric):
 
         for ix, (x, y, a, s) in iterator:
 
-            if self.abs:
-                a = np.abs(a)
-
             if self.normalise:
                 a = self.normalise_func(a)
 
+            if self.abs:
+                a = np.abs(a)
+
             s = s.flatten()
             s = s.astype(bool)
             a = a.flatten()
