@@ -1,5 +1,4 @@
 """This module contains the collection of randomisation metrics to evaluate attribution-based explanations of neural network models."""
-import random
 import warnings
 from typing import Callable, Dict, List, Union
 
@@ -135,16 +134,17 @@ class ModelParameterRandomisation(Metric):
 
             # Initialise the metric and evaluate explanations by calling the metric instance.
             >> metric = ModelParameterRandomisation(abs=True, normalise=False)
-            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{}}
+            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{})
         """
-        # Reshape input batch to channel first order:
+
+        # Reshape input batch to channel first order.
         if "channel_first" in kwargs and isinstance(kwargs["channel_first"], bool):
             channel_first = kwargs.get("channel_first")
         else:
             channel_first = utils.infer_channel_first(x_batch)
         x_batch_s = utils.make_channel_first(x_batch, channel_first)
 
-        # Wrap the model into an interface
+        # Wrap the model into an interface.
         if model:
             model = utils.get_wrapped_model(model, channel_first)
 
@@ -172,6 +172,8 @@ class ModelParameterRandomisation(Metric):
                 targets=y_batch,
                 **self.kwargs,
             )
+
+        # Expand attributions to input dimensionality.
         a_batch = utils.expand_attribution_channel(a_batch, x_batch_s)
 
         # Asserts.
@@ -200,14 +202,14 @@ class ModelParameterRandomisation(Metric):
 
             for ix, (a, a_per) in enumerate(zip(a_batch, a_perturbed)):
 
-                if self.abs:
-                    a = np.abs(a)
-                    a_per = np.abs(a_per)
-
                 if self.normalise:
                     a = self.normalise_func(a)
                     a_per = self.normalise_func(a_per)
 
+                if self.abs:
+                    a = np.abs(a)
+                    a_per = np.abs(a_per)
+
                 # Compute distance measure.
                 similarity = self.similarity_func(a_per.flatten(), a.flatten())
 
@@ -337,16 +339,17 @@ class RandomLogit(Metric):
 
             # Initialise the metric and evaluate explanations by calling the metric instance.
             >> metric = RandomLogit(abs=True, normalise=False)
-            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{}}
+            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{})
         """
-        # Reshape input batch to channel first order:
+
+        # Reshape input batch to channel first order.
         if "channel_first" in kwargs and isinstance(kwargs["channel_first"], bool):
             channel_first = kwargs.get("channel_first")
         else:
             channel_first = utils.infer_channel_first(x_batch)
         x_batch_s = utils.make_channel_first(x_batch, channel_first)
 
-        # Wrap the model into an interface
+        # Wrap the model into an interface.
         if model:
             model = utils.get_wrapped_model(model, channel_first)
 
@@ -373,12 +376,14 @@ class RandomLogit(Metric):
                 targets=y_batch,
                 **self.kwargs,
             )
+
+        # Expand attributions to input dimensionality.
         a_batch = utils.expand_attribution_channel(a_batch, x_batch_s)
 
         # Asserts.
         asserts.assert_attributions(x_batch=x_batch, a_batch=a_batch)
 
-        # use tqdm progressbar if not disabled
+        # Use tqdm progressbar if not disabled.
         if not self.display_progressbar:
             iterator = enumerate(zip(x_batch_s, y_batch, a_batch))
         else:
@@ -388,17 +393,17 @@ class RandomLogit(Metric):
 
         for ix, (x, y, a) in iterator:
 
-            if self.abs:
-                a = np.abs(a)
-
             if self.normalise:
                 a = self.normalise_func(a)
 
+            if self.abs:
+                a = np.abs(a)
+
             # Randomly select off-class labels.
-            random.seed(a=self.seed)
+            np.random.seed(self.seed)
             y_off = np.array(
                 [
-                    random.choice(
+                    np.random.choice(
                         [y_ for y_ in list(np.arange(0, self.num_classes)) if y_ != y]
                     )
                 ]
@@ -412,12 +417,12 @@ class RandomLogit(Metric):
                 **self.kwargs,
             )
 
-            if self.abs:
-                a_perturbed = np.abs(a_perturbed)
-
             if self.normalise:
                 a_perturbed = self.normalise_func(a_perturbed)
 
+            if self.abs:
+                a_perturbed = np.abs(a_perturbed)
+
             self.last_results.append(
                 self.similarity_func(a.flatten(), a_perturbed.flatten())
             )
