@@ -158,9 +158,10 @@ class LocalLipschitzEstimate(Metric):
 
             # Initialise the metric and evaluate explanations by calling the metric instance.
             >> metric = LocalLipschitzEstimate(abs=True, normalise=False)
-            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{}}
+            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{})
         """
-        # Reshape input batch to channel first order:
+
+        # Reshape input batch to channel first order.
         if "channel_first" in kwargs and isinstance(kwargs["channel_first"], bool):
             channel_first = kwargs.get("channel_first")
         else:
@@ -195,12 +196,14 @@ class LocalLipschitzEstimate(Metric):
                 targets=y_batch,
                 **self.kwargs,
             )
+
+        # Expand attributions to input dimensionality.
         a_batch = utils.expand_attribution_channel(a_batch, x_batch_s)
 
         # Get explanation function and make asserts.
         asserts.assert_attributions(x_batch=x_batch_s, a_batch=a_batch)
 
-        # use tqdm progressbar if not disabled
+        # Use tqdm progressbar if not disabled.
         if not self.display_progressbar:
             iterator = enumerate(zip(x_batch_s, y_batch, a_batch))
         else:
@@ -210,18 +213,20 @@ class LocalLipschitzEstimate(Metric):
 
         for ix, (x, y, a) in iterator:
 
-            if self.abs:
-                a = np.abs(a)
-
             if self.normalise:
                 a = self.normalise_func(a)
 
+            if self.abs:
+                a = np.abs(a)
+
             similarity_max = 0.0
             for i in range(self.nr_samples):
 
                 # Perturb input.
                 x_perturbed = self.perturb_func(
                     arr=x,
+                    indices=np.arange(0, x.size),
+                    indexed_axes=np.arange(0, x.ndim),
                     **self.kwargs,
                 )
                 x_input = model.shape_input(x_perturbed, x.shape, channel_first=True)
@@ -235,11 +240,12 @@ class LocalLipschitzEstimate(Metric):
                     **self.kwargs,
                 )
 
-                if self.abs:
-                    a_perturbed = np.abs(a_perturbed)
                 if self.normalise:
                     a_perturbed = self.normalise_func(a_perturbed)
 
+                if self.abs:
+                    a_perturbed = np.abs(a_perturbed)
+
                 # Measure similarity.
                 similarity = self.similarity_func(
                     a=a.flatten(),
@@ -287,11 +293,12 @@ class MaxSensitivity(Metric):
             default_plot_func (callable): Callable that plots the metrics result.
             disable_warnings (boolean): Indicates whether the warnings are printed, default=False.
             display_progressbar (boolean): Indicates whether a tqdm-progress-bar is printed, default=False.
-            perturb_radius (float): Perturbation radius, default=0.2.
+            lower_bound (float): Lower Bound of Perturbation, default=0.2.
+            upper_bound (None, float): Upper Bound of Perturbation, default=None.
             nr_samples (integer): The number of samples iterated, default=200.
             norm_numerator (callable): Function for norm calculations on the numerator, default=fro_norm.
             norm_denominator (callable): Function for norm calculations on the denominator, default=fro_norm.
-            perturb_func (callable): Input perturbation function, default=uniform_sampling.
+            perturb_func (callable): Input perturbation function, default=uniform_noise.
             similarity_func (callable): Similarity function applied to compare input and perturbed input,
             default=difference.
         """
@@ -308,10 +315,9 @@ class MaxSensitivity(Metric):
         self.nr_samples = self.kwargs.get("nr_samples", 200)
         self.norm_numerator = self.kwargs.get("norm_numerator", fro_norm)
         self.norm_denominator = self.kwargs.get("norm_denominator", fro_norm)
-        self.perturb_func = self.kwargs.get(
-            "perturb_func", perturb_func.uniform_sampling
-        )
-        self.perturb_radius = self.kwargs.get("perturb_radius", 0.2)
+        self.perturb_func = self.kwargs.get("perturb_func", perturb_func.uniform_noise)
+        self.lower_bound = self.kwargs.get("lower_bound", 0.2)
+        self.upper_bound = self.kwargs.get("upper_bound", None)
         self.similarity_func = self.kwargs.get(
             "similarity_func", similar_func.difference
         )
@@ -324,7 +330,7 @@ class MaxSensitivity(Metric):
             warn_func.warn_parameterisation(
                 metric_name=self.__class__.__name__,
                 sensitive_params=(
-                    "amount of noise added 'perturb_radius', the number of samples "
+                    "amount of noise added 'lower_bound' and 'upper_bound', the number of samples "
                     "iterated over 'nr_samples', the function to perturb the input "
                     "'perturb_func', the similarity metric 'similarity_func' as well as "
                     "norm calculations on the numerator and denominator of the sensitivity"
@@ -335,7 +341,7 @@ class MaxSensitivity(Metric):
                     ".' arXiv preprint arXiv:1901.09392 (2019)"
                 ),
             )
-            warn_func.warn_noise_zero(noise=self.perturb_radius)
+            warn_func.warn_noise_zero(noise=self.lower_bound)
 
     def __call__(
         self,
@@ -389,9 +395,10 @@ class MaxSensitivity(Metric):
 
             # Initialise the metric and evaluate explanations by calling the metric instance.
             >> metric = MaxSensitivity(abs=True, normalise=False)
-            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{}}
+            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{})
         """
-        # Reshape input batch to channel first order:
+
+        # Reshape input batch to channel first order.
         if "channel_first" in kwargs and isinstance(kwargs["channel_first"], bool):
             channel_first = kwargs.get("channel_first")
         else:
@@ -426,12 +433,14 @@ class MaxSensitivity(Metric):
                 targets=y_batch,
                 **self.kwargs,
             )
+
+        # Expand attributions to input dimensionality.
         a_batch = utils.expand_attribution_channel(a_batch, x_batch_s)
 
         # Get explanation function and make asserts.
         asserts.assert_attributions(x_batch=x_batch_s, a_batch=a_batch)
 
-        # use tqdm progressbar if not disabled
+        # Use tqdm progressbar if not disabled.
         if not self.display_progressbar:
             iterator = enumerate(zip(x_batch_s, y_batch, a_batch))
         else:
@@ -441,18 +450,20 @@ class MaxSensitivity(Metric):
 
         for ix, (x, y, a) in iterator:
 
-            if self.abs:
-                a = np.abs(a)
-
             if self.normalise:
                 a = self.normalise_func(a)
 
+            if self.abs:
+                a = np.abs(a)
+
             sensitivities_norm_max = 0.0
             for _ in range(self.nr_samples):
 
                 # Perturb input.
                 x_perturbed = self.perturb_func(
                     arr=x,
+                    indices=np.arange(0, x.size),
+                    indexed_axes=np.arange(0, x.ndim),
                     **self.kwargs,
                 )
                 x_input = model.shape_input(x_perturbed, x.shape, channel_first=True)
@@ -466,12 +477,12 @@ class MaxSensitivity(Metric):
                     **self.kwargs,
                 )
 
-                if self.abs:
-                    a_perturbed = np.abs(a_perturbed)
-
                 if self.normalise:
                     a_perturbed = self.normalise_func(a_perturbed)
 
+                if self.abs:
+                    a_perturbed = np.abs(a_perturbed)
+
                 # Measure sensitivity.
                 sensitivities = self.similarity_func(
                     a=a.flatten(), b=a_perturbed.flatten()
@@ -520,11 +531,12 @@ class AvgSensitivity(Metric):
             default_plot_func (callable): Callable that plots the metrics result.
             disable_warnings (boolean): Indicates whether the warnings are printed, default=False.
             display_progressbar (boolean): Indicates whether a tqdm-progress-bar is printed, default=False.
-            perturb_radius (float): Perturbation radius, default=0.2.
+            lower_bound (float): Lower Bound of Perturbation, default=0.2.
+            upper_bound (None, float): Upper Bound of Perturbation, default=None.
             nr_samples (integer): The number of samples iterated, default=200.
             norm_numerator (callable): Function for norm calculations on the numerator, default=fro_norm.
             norm_denominator (callable): Function for norm calculations on the denominator, default=fro_norm.
-            perturb_func (callable): Input perturbation function, default=uniform_sampling.
+            perturb_func (callable): Input perturbation function, default=uniform_noise.
             similarity_func (callable): Similarity function applied to compare input and perturbed input,
             default=difference.
         """
@@ -541,10 +553,9 @@ class AvgSensitivity(Metric):
         self.nr_samples = self.kwargs.get("nr_samples", 200)
         self.norm_numerator = self.kwargs.get("norm_numerator", fro_norm)
         self.norm_denominator = self.kwargs.get("norm_denominator", fro_norm)
-        self.perturb_func = self.kwargs.get(
-            "perturb_func", perturb_func.uniform_sampling
-        )
-        self.perturb_radius = self.kwargs.get("perturb_radius", 0.2)
+        self.perturb_func = self.kwargs.get("perturb_func", perturb_func.uniform_noise)
+        self.lower_bound = self.kwargs.get("lower_bound", 0.2)
+        self.upper_bound = self.kwargs.get("upper_bound", None)
         self.similarity_func = self.kwargs.get(
             "similarity_func", similar_func.difference
         )
@@ -556,7 +567,7 @@ class AvgSensitivity(Metric):
             warn_func.warn_parameterisation(
                 metric_name=self.__class__.__name__,
                 sensitive_params=(
-                    "amount of noise added 'perturb_radius', the number of samples "
+                    "amount of noise added 'lower_bound' and 'upper_bound', the number of samples "
                     "iterated over 'nr_samples', the function to perturb the input "
                     "'perturb_func', the similarity metric 'similarity_func' as well as "
                     "norm calculations on the numerator and denominator of the sensitivity"
@@ -567,7 +578,7 @@ class AvgSensitivity(Metric):
                     ".' arXiv preprint arXiv:1901.09392 (2019)"
                 ),
             )
-            warn_func.warn_noise_zero(noise=self.perturb_radius)
+            warn_func.warn_noise_zero(noise=self.lower_bound)
 
     def __call__(
         self,
@@ -621,9 +632,10 @@ class AvgSensitivity(Metric):
 
             # Initialise the metric and evaluate explanations by calling the metric instance.
             >> metric = AvgSensitivity(abs=True, normalise=False)
-            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{}}
+            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{})
         """
-        # Reshape input batch to channel first order:
+
+        # Reshape input batch to channel first order.
         if "channel_first" in kwargs and isinstance(kwargs["channel_first"], bool):
             channel_first = kwargs.get("channel_first")
         else:
@@ -658,12 +670,14 @@ class AvgSensitivity(Metric):
                 targets=y_batch,
                 **self.kwargs,
             )
+
+        # Expand attributions to input dimensionality.
         a_batch = utils.expand_attribution_channel(a_batch, x_batch_s)
 
         # Asserts.
         asserts.assert_attributions(x_batch=x_batch_s, a_batch=a_batch)
 
-        # use tqdm progressbar if not disabled
+        # Use tqdm progressbar if not disabled.
         if not self.display_progressbar:
             iterator = enumerate(zip(x_batch_s, y_batch, a_batch))
         else:
@@ -673,18 +687,20 @@ class AvgSensitivity(Metric):
 
         for ix, (x, y, a) in iterator:
 
-            if self.abs:
-                a = np.abs(a)
-
             if self.normalise:
                 a = self.normalise_func(a)
 
+            if self.abs:
+                a = np.abs(a)
+
             self.sub_results = []
             for _ in range(self.nr_samples):
 
                 # Perturb input.
                 x_perturbed = self.perturb_func(
                     arr=x,
+                    indices=np.arange(0, x.size),
+                    indexed_axes=np.arange(0, x.ndim),
                     **self.kwargs,
                 )
                 x_input = model.shape_input(x_perturbed, x.shape, channel_first=True)
@@ -698,12 +714,12 @@ class AvgSensitivity(Metric):
                     **self.kwargs,
                 )
 
-                if self.abs:
-                    a_perturbed = np.abs(a_perturbed)
-
                 if self.normalise:
                     a_perturbed = self.normalise_func(a_perturbed)
 
+                if self.abs:
+                    a_perturbed = np.abs(a_perturbed)
+
                 sensitivities = self.similarity_func(
                     a=a.flatten(), b=a_perturbed.flatten()
                 )
@@ -758,6 +774,7 @@ class Continuity(Metric):
             perturb_func (callable): Input perturbation function, default=translation_x_direction.
             similarity_func (callable): Similarity function applied to compare input and perturbed input,
             default=lipschitz_constant.
+            softmax (boolean): Indicates wheter to use softmax probabilities or logits in model prediction.
         """
         super().__init__()
 
@@ -775,9 +792,11 @@ class Continuity(Metric):
         self.perturb_func = self.kwargs.get(
             "perturb_func", perturb_func.translation_x_direction
         )
+        self.perturb_baseline = self.kwargs.get("perturb_baseline", "black")
         self.similarity_func = self.kwargs.get(
             "similarity_func", similar_func.lipschitz_constant
         )
+        self.softmax = self.kwargs.get("softmax", False)
         self.last_results = []
         self.all_results = []
 
@@ -850,9 +869,10 @@ class Continuity(Metric):
 
             # Initialise the metric and evaluate explanations by calling the metric instance.
             >> metric = Continuity(abs=True, normalise=False)
-            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{}}
+            >> scores = metric(model=model, x_batch=x_batch, y_batch=y_batch, a_batch=a_batch_saliency, **{})
         """
-        # Reshape input batch to channel first order:
+
+        # Reshape input batch to channel first order.
         if "channel_first" in kwargs and isinstance(kwargs["channel_first"], bool):
             channel_first = kwargs.get("channel_first")
         else:
@@ -887,20 +907,23 @@ class Continuity(Metric):
                 targets=y_batch,
                 **self.kwargs,
             )
+
+        # Expand attributions to input dimensionality and infer input dimensions covered by the attributions
         a_batch = utils.expand_attribution_channel(a_batch, x_batch_s)
+        a_axes = utils.infer_attribution_axes(a_batch, x_batch_s)
 
         # Asserts.
         asserts.assert_patch_size(patch_size=self.patch_size, shape=x_batch_s.shape[2:])
         asserts.assert_attributions(x_batch=x_batch_s, a_batch=a_batch)
 
-        # Get number of patches for input shape (ignore batch and channel dim)
+        # Get number of patches for input shape (ignore batch and channel dim).
         self.nr_patches = utils.get_nr_patches(
             patch_size=self.patch_size,
             shape=x_batch_s.shape[2:],
             overlap=True,
         )
 
-        # use tqdm progressbar if not disabled
+        # Use tqdm progressbar if not disabled.
         if not self.display_progressbar:
             iterator = enumerate(zip(x_batch_s, y_batch, a_batch))
         else:
@@ -911,12 +934,12 @@ class Continuity(Metric):
         self.dx = np.prod(x_batch_s.shape[2:]) // self.nr_steps
         for ix, (x, y, a) in iterator:
 
-            if self.abs:
-                a = np.abs(a)
-
             if self.normalise:
                 a = self.normalise_func(a)
 
+            if self.abs:
+                a = np.abs(a)
+
             sub_results = {k: [] for k in range(self.nr_patches + 1)}
 
             for step in range(self.nr_steps):
@@ -925,6 +948,8 @@ class Continuity(Metric):
                 dx_step = (step + 1) * self.dx
                 x_perturbed = self.perturb_func(
                     arr=x,
+                    indices=np.arange(0, x.size),
+                    indexed_axes=np.arange(0, x.ndim),
                     perturb_dx=dx_step,
                     **self.kwargs,
                 )
@@ -937,25 +962,24 @@ class Continuity(Metric):
                     targets=y,
                     **self.kwargs,
                 )
-                a_perturbed = utils.expand_attribution_channel(a_batch, x_batch_s)
-
-                if self.abs:
-                    a_perturbed = np.abs(a_perturbed)
+                # Taking the first element, since a_perturbed will be expanded to a batch dimension
+                # not expected by the current index management functions
+                a_perturbed = utils.expand_attribution_channel(a_perturbed, x_input)[0]
 
                 if self.normalise:
                     a_perturbed = self.normalise_func(a_perturbed)
 
+                if self.abs:
+                    a_perturbed = np.abs(a_perturbed)
+
                 # Store the prediction score as the last element of the sub_self.last_results dictionary.
-                y_pred = float(
-                    model.predict(x_input, softmax_act=False, **self.kwargs)[:, y]
-                )
+                y_pred = float(model.predict(x_input, **self.kwargs)[:, y])
 
                 sub_results[self.nr_patches].append(y_pred)
 
                 # create patches by splitting input into grid
                 axis_iterators = [
-                    range(0, x_input.shape[axis], self.patch_size)
-                    for axis in range(1, x_input.ndim)
+                    range(0, x_input.shape[axis], self.patch_size) for axis in a_axes
                 ]
                 for ix_patch, top_left_coords in enumerate(
                     itertools.product(*axis_iterators)
@@ -965,18 +989,20 @@ class Continuity(Metric):
                     patch_slice = utils.create_patch_slice(
                         patch_size=self.patch_size,
                         coords=top_left_coords,
-                        expand_first_dim=True,
                     )
 
-                    a_perturbed_patch = a_perturbed[patch_slice]
-                    if self.abs:
-                        a_perturbed_patch = np.abs(a_perturbed_patch.flatten())
+                    a_perturbed_patch = a_perturbed[
+                        utils.expand_indices(a_perturbed, patch_slice, a_axes)
+                    ]
 
                     if self.normalise:
                         a_perturbed_patch = self.normalise_func(
                             a_perturbed_patch.flatten()
                         )
 
+                    if self.abs:
+                        a_perturbed_patch = np.abs(a_perturbed_patch.flatten())
+
                     # Sum attributions for patch.
                     patch_sum = float(sum(a_perturbed_patch))
                     sub_results[ix_patch].append(patch_sum)
