@@ -118,6 +118,71 @@ if util.find_spec("zennit"):
             },
             {"max": 0},
         ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "canonizer": None,
+                "composite": "guided_backprop",
+                "attributor": zattr.Gradient,
+                "neg_only": True,
+                "xai_lib": "zennit",
+                "reduce_axes": (1,),
+            },
+            {"shape": (1, 28, 28)},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "canonizer": None,
+                "composite": "guided_backprop",
+                "attributor": zattr.Gradient,
+                "neg_only": True,
+                "xai_lib": "zennit",
+                "reduce_axes": (1, 2),
+            },
+            {"shape": (1, 1, 28)},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "canonizer": None,
+                "composite": "guided_backprop",
+                "attributor": zattr.Gradient,
+                "neg_only": True,
+                "xai_lib": "zennit",
+                "reduce_axes": (3,),
+            },
+            {"shape": (1, 28, 1)},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "canonizer": None,
+                "composite": "guided_backprop",
+                "attributor": zattr.Gradient,
+                "neg_only": True,
+                "xai_lib": "zennit",
+                "reduce_axes": (0, 1),
+            },
+            {"exception": AssertionError},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "canonizer": None,
+                "composite": "guided_backprop",
+                "attributor": zattr.Gradient,
+                "neg_only": True,
+                "xai_lib": "zennit",
+                "reduce_axes": (1, 2, 3, 4, 5),
+            },
+            {"exception": AssertionError},
+        ),
     ]
 else:
     zennit_tests = []
@@ -327,6 +392,22 @@ else:
             },
             {"value": 0.0},
         ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "method": "Control Var. Random Uniform",
+            },
+            {"min": 0.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d_no_abatch"),
+            {
+                "method": "Control Var. Random Uniform",
+            },
+            {"min": 0.0, "max": 1.0},
+        ),
         (
             lazy_fixture("load_1d_3ch_conv_model"),
             lazy_fixture("almost_uniform_1d_no_abatch"),
@@ -609,6 +690,66 @@ else:
             },
             {"min": 0.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "method": "Gradient",
+                "normalise": True,
+                "abs": True,
+                "normalise_func": normalise_by_max,
+                "reduce_axes": (1,),
+            },
+            {"shape": (1, 28, 28)},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "method": "Gradient",
+                "normalise": True,
+                "abs": True,
+                "normalise_func": normalise_by_max,
+                "reduce_axes": (1, 2),
+            },
+            {"shape": (1, 1, 28)},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "method": "Gradient",
+                "normalise": True,
+                "abs": True,
+                "normalise_func": normalise_by_max,
+                "reduce_axes": (3,),
+            },
+            {"shape": (1, 28, 1)},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "method": "Gradient",
+                "normalise": True,
+                "abs": True,
+                "normalise_func": normalise_by_max,
+                "reduce_axes": (0, 1),
+            },
+            {"exception": AssertionError},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "method": "Gradient",
+                "normalise": True,
+                "abs": True,
+                "normalise_func": normalise_by_max,
+                "reduce_axes": (1, 2, 3, 4, 5, 6),
+            },
+            {"exception": AssertionError},
+        ),
         # tf-explain
         (
             lazy_fixture("load_mnist_model_tf"),
@@ -649,17 +790,21 @@ else:
         (
             lazy_fixture("load_mnist_model_tf"),
             lazy_fixture("load_mnist_images_tf"),
-            {
-            },
+            {},
             {"warning": UserWarning},
         ),
         (
             None,
             lazy_fixture("load_mnist_images_tf"),
-            {
-            },
+            {},
             {"exception": ValueError},
         ),
+        (
+            lazy_fixture("load_mnist_model_tf"),
+            lazy_fixture("load_mnist_images_tf"),
+            {"method": "InputXGradient", "abs": True, "reduce_axes": (0, 1, 2)},
+            {"exception": KeyError},
+        ),
     ],
 )
 def test_explain_func(
@@ -691,6 +836,8 @@ def test_explain_func(
             assert all(
                 s == expected["value"] for s in a_batch.flatten()
             ), "Test failed."
+        elif "shape" in expected:
+            assert all(s.shape == expected["shape"] for s in a_batch), "Test failed."
         elif "warning" in expected:
             with pytest.warns(expected["warning"]):
                 a_batch = explain(
@@ -979,7 +1126,8 @@ def test_get_explanation(
     if "exception" in expected:
         with pytest.raises(expected["exception"]):
             a_batch = get_explanation(
-                model=model, inputs=x_batch, targets=y_batch, **params)
+                model=model, inputs=x_batch, targets=y_batch, **params
+            )
         return
 
     a_batch = get_explanation(model=model, inputs=x_batch, targets=y_batch, **params)
