@@ -17,62 +17,62 @@ def segmentation_setup():
 
 @pytest.fixture
 def baseline_none_1d():
-    return {"choice": None, "arr": np.random.uniform(0, 1, size=(1, 3, 224))}
+    return {"value": None, "arr": np.random.uniform(0, 1, size=(128,))}
 
 
 @pytest.fixture
-def baseline_none_2d():
-    return {"choice": None, "arr": np.random.uniform(0, 1, size=(1, 3, 224))}
+def baseline_none_3d():
+    return {"value": None, "arr": np.random.uniform(0, 1, size=(128, 64, 32))}
 
 
 @pytest.fixture
 def baseline_black_1d():
-    return {"choice": "black", "arr": np.random.uniform(0, 1, size=(1, 3, 224))}
+    return {"value": "black", "arr": np.random.uniform(0, 1, size=(128,))}
 
 
 @pytest.fixture
-def baseline_black_2d():
-    return {"choice": "black", "arr": np.random.uniform(0, 1, size=(1, 3, 224))}
+def baseline_black_3d():
+    return {"value": "black", "arr": np.random.uniform(0, 1, size=(128, 64, 32))}
 
 
 @pytest.fixture
 def baseline_uniform_1d():
-    return {"choice": "uniform", "arr": np.random.uniform(0, 1, size=(1, 3, 224))}
+    return {"value": "uniform", "arr": np.random.uniform(0, 1, size=(128,))}
 
 
 @pytest.fixture
-def baseline_uniform_2d():
-    return {"choice": "uniform", "arr": np.random.uniform(0, 1, size=(1, 3, 224))}
+def baseline_uniform_3d():
+    return {"value": "uniform", "arr": np.random.uniform(0, 1, size=(128, 64, 32))}
 
 
 @pytest.fixture
 def baseline_random_1d():
-    return {"choice": "random", "arr": np.random.uniform(0, 1, size=(1, 3, 224))}
+    return {"value": "random", "arr": np.random.uniform(0, 1, size=(128,))}
 
 
 @pytest.fixture
-def baseline_random_2d():
-    return {"choice": "random", "arr": np.random.uniform(0, 1, size=(1, 3, 224))}
+def baseline_random_3d():
+    return {"value": "random", "arr": np.random.uniform(0, 1, size=(128, 64, 32))}
 
 
 @pytest.fixture
 def baseline_white_1d():
-    return {"choice": "white", "arr": np.random.uniform(0, 1, size=(1, 3, 224))}
+    return {"value": "white", "arr": np.random.uniform(0, 1, size=(128,))}
 
 
 @pytest.fixture
-def baseline_white_2d():
-    return {"choice": "white", "arr": np.random.uniform(0, 1, size=(1, 3, 224))}
+def baseline_white_3d():
+    return {"value": "white", "arr": np.random.uniform(0, 1, size=(128, 64, 32))}
 
 
 @pytest.fixture
 def baseline_mean_1d():
-    return {"choice": "mean", "arr": np.random.uniform(0, 1, size=(1, 3, 2222))}
+    return {"value": "mean", "arr": np.random.uniform(0, 1, size=(128,))}
 
 
 @pytest.fixture
-def baseline_mean_2d():
-    return {"choice": "mean", "arr": np.random.uniform(0, 1, size=(1, 3, 2222, 2222))}
+def baseline_mean_3d():
+    return {"value": "mean", "arr": np.random.uniform(0, 1, size=(128, 64, 32))}
 
 
 @pytest.fixture
@@ -130,12 +130,8 @@ def mock_same_input_2d():
     return {"x": np.zeros((1, 28, 28, 28))}
 
 
-def random_input(c_in, imsize):
-    return np.random.uniform(0, 1, size=(c_in, imsize, imsize))
-
-
-def random_kernel(c_in, c_out, groups, ksize):
-    return np.random.uniform(0, 1, size=(c_out, c_in // groups, ksize, ksize))
+def random_array(shape):
+    return np.random.uniform(0, 1, size=shape)
 
 
 @pytest.mark.utils
@@ -176,37 +172,70 @@ def test_get_superpixel_segments(
 
 @pytest.mark.utils
 @pytest.mark.parametrize(
-    "data,expected",
+    "data,shape,expected",
     [
-        (lazy_fixture("baseline_black_1d"), {"value": 0.0}),
-        (lazy_fixture("baseline_black_2d"), {"value": 0.0}),
-        (lazy_fixture("baseline_white_1d"), {"value": 1.0}),
-        (lazy_fixture("baseline_white_2d"), {"value": 1.0}),
-        (lazy_fixture("baseline_mean_1d"), {"value": 0.5}),
-        (lazy_fixture("baseline_mean_2d"), {"value": 0.5}),
-        (lazy_fixture("baseline_none_1d"), {"exception": AssertionError}),
-        (lazy_fixture("baseline_none_2d"), {"exception": AssertionError}),
+        (lazy_fixture("baseline_black_1d"), {"return_shape": (1,)}, {"value": 0.0}),
+        (lazy_fixture("baseline_black_1d"), {"return_shape": (3, 4)}, {"value": 0.0}),
+        (lazy_fixture("baseline_black_3d"), {"return_shape": (1,)}, {"value": 0.0}),
+        (lazy_fixture("baseline_black_3d"), {"return_shape": (3, 4)}, {"value": 0.0}),
+        (lazy_fixture("baseline_white_1d"), {"return_shape": (1,)}, {"value": 1.0}),
+        (lazy_fixture("baseline_white_1d"), {"return_shape": (3, 4)}, {"value": 1.0}),
+        (lazy_fixture("baseline_white_3d"), {"return_shape": (1,)}, {"value": 1.0}),
+        (lazy_fixture("baseline_white_3d"), {"return_shape": (3, 4)}, {"value": 1.0}),
+        (lazy_fixture("baseline_mean_1d"), {"return_shape": (1,)}, {"value": 0.5}),
+        (lazy_fixture("baseline_mean_1d"), {"return_shape": (3, 4)}, {"value": 0.5}),
+        (lazy_fixture("baseline_mean_3d"), {"return_shape": (1,)}, {"value": 0.5}),
+        (lazy_fixture("baseline_mean_3d"), {"return_shape": (3, 4)}, {"value": 0.5}),
+        (
+            lazy_fixture("baseline_none_1d"),
+            {"return_shape": (1,)},
+            {"exception": ValueError},
+        ),
+        (
+            lazy_fixture("baseline_none_1d"),
+            {"return_shape": (3, 4)},
+            {"exception": ValueError},
+        ),
+        (
+            lazy_fixture("baseline_none_3d"),
+            {"return_shape": (1,)},
+            {"exception": ValueError},
+        ),
+        (
+            lazy_fixture("baseline_none_3d"),
+            {"return_shape": (3, 4)},
+            {"exception": ValueError},
+        ),
     ],
 )
-def test_get_baseline_value(data: np.ndarray, expected: Union[float, dict, bool]):
+def test_get_baseline_value(
+    data: np.ndarray, shape: Tuple, expected: Union[float, dict, bool]
+):
     if "exception" in expected:
         with pytest.raises(expected["exception"]):
-            get_baseline_value(choice=data["choice"], arr=data["arr"])
+            get_baseline_value(
+                value=data["value"], return_shape=shape["return_shape"], arr=data["arr"]
+            )
         return
-    out = get_baseline_value(choice=data["choice"], arr=data["arr"])
-    assert out == pytest.approx(expected["value"], rel=1e-1, abs=1e-1), "Test failed."
+    out = get_baseline_value(
+        value=data["value"], return_shape=shape["return_shape"], arr=data["arr"]
+    )
+    assert (
+        out == pytest.approx(expected["value"], rel=1e-1, abs=1e-1)
+        and out.shape == shape["return_shape"]
+    ), "Test failed."
 
 
 @pytest.mark.utils
 @pytest.mark.parametrize(
     "data,expected",
     [
-        (lazy_fixture("baseline_black_2d"), dict),
-        (lazy_fixture("baseline_black_2d"), dict),
-        (lazy_fixture("baseline_white_2d"), dict),
-        (lazy_fixture("baseline_white_2d"), dict),
-        (lazy_fixture("baseline_mean_2d"), dict),
-        (lazy_fixture("baseline_mean_2d"), dict),
+        (lazy_fixture("baseline_black_3d"), dict),
+        (lazy_fixture("baseline_black_3d"), dict),
+        (lazy_fixture("baseline_white_3d"), dict),
+        (lazy_fixture("baseline_white_3d"), dict),
+        (lazy_fixture("baseline_mean_3d"), dict),
+        (lazy_fixture("baseline_mean_3d"), dict),
     ],
 )
 def test_get_baseline_dict(data: np.ndarray, expected: Union[float, dict, bool]):
@@ -398,158 +427,63 @@ def test_get_wrapped_model(
     [
         (
             {
-                "c_in": 3,
-                "c_out": 3,
-                "imsize": 224,
-                "kgroups": 3,
-                "ksize": 8,
-                "stride": 1,
-                "padding": 0,
-                "groups": 3,
-                "pad_output": False,
+                "arr_shape": (10, 20, 30, 40),
+                "kernel_shape": (3, 3),
+                "indices": [15, 22, 30],
+                "indexed_axes": [2, 3],
             },
-            {"shape": (3, 217, 217)},
+            {"shape": (10, 20, 30, 40)},
         ),
         (
             {
-                "c_in": 3,
-                "c_out": 3,
-                "imsize": 224,
-                "kgroups": 3,
-                "ksize": 8,
-                "stride": 1,
-                "padding": 3,
-                "groups": 3,
-                "pad_output": False,
+                "arr_shape": (10, 20, 30, 40),
+                "kernel_shape": (4, 9, 21),
+                "indices": [15, 22, 30],
+                "indexed_axes": [0, 1, 2],
             },
-            {"shape": (3, 223, 223)},
+            {"shape": (10, 20, 30, 40)},
         ),
         (
             {
-                "c_in": 3,
-                "c_out": 3,
-                "imsize": 224,
-                "kgroups": 3,
-                "ksize": 8,
-                "stride": 3,
-                "padding": 0,
-                "groups": 3,
-                "pad_output": False,
+                "arr_shape": (10, 20, 30, 40),
+                "kernel_shape": (1, 1, 1),
+                "indices": [15, 22, 30],
+                "indexed_axes": [1, 2, 3],
             },
-            {"shape": (3, 73, 73)},
+            {"shape": (10, 20, 30, 40)},
         ),
         (
             {
-                "c_in": 6,
-                "c_out": 3,
-                "imsize": 224,
-                "kgroups": 3,
-                "ksize": 8,
-                "stride": 1,
-                "padding": 0,
-                "groups": 3,
-                "pad_output": False,
-            },
-            {"shape": (3, 217, 217)},
-        ),
-        (
-            {
-                "c_in": 6,
-                "c_out": 3,
-                "imsize": 224,
-                "kgroups": 2,
-                "ksize": 8,
-                "stride": 1,
-                "padding": 0,
-                "groups": 3,
-                "pad_output": False,
+                "arr_shape": (10, 20, 30, 40),
+                "kernel_shape": (1, 3, 3),
+                "indices": [15, 22, 30],
+                "indexed_axes": [2, 3],
             },
             {"exception": AssertionError},
         ),
-        (
-            {
-                "c_in": 3,
-                "c_out": 3,
-                "imsize": 224,
-                "kgroups": 3,
-                "ksize": 8,
-                "stride": 1,
-                "padding": 0,
-                "groups": 3,
-                "pad_output": True,
-            },
-            {"shape": (3, 224, 224)},
-        ),
-        (
-            {
-                "c_in": 3,
-                "c_out": 3,
-                "imsize": 224,
-                "kgroups": 3,
-                "ksize": 8,
-                "stride": 1,
-                "padding": 3,
-                "groups": 3,
-                "pad_output": True,
-            },
-            {"exception": NotImplementedError},
-        ),
-        (
-            {
-                "c_in": 3,
-                "c_out": 3,
-                "imsize": 224,
-                "kgroups": 3,
-                "ksize": 8,
-                "stride": 3,
-                "padding": 0,
-                "groups": 3,
-                "pad_output": True,
-            },
-            {"exception": NotImplementedError},
-        ),
-        (
-            {
-                "c_in": 6,
-                "c_out": 3,
-                "imsize": 224,
-                "kgroups": 3,
-                "ksize": 8,
-                "stride": 1,
-                "padding": 0,
-                "groups": 3,
-                "pad_output": True,
-            },
-            {"shape": (3, 224, 224)},
-        ),
     ],
 )
-def test_conv2D_numpy(
+def test_blur_at_indices(
     params: dict,
     expected: Union[float, dict, bool],
 ):
-    input = random_input(params["c_in"], params["imsize"])
-    kernel = random_kernel(
-        params["c_in"], params["c_out"], params["kgroups"], params["ksize"]
-    )
+    input = random_array(params["arr_shape"])
+    kernel = random_array(params["kernel_shape"])
+
     if "exception" in expected:
         with pytest.raises(expected["exception"]):
-            out = conv2D_numpy(
-                x=input,
+            out = blur_at_indices(
+                arr=input,
                 kernel=kernel,
-                stride=params["stride"],
-                padding=params["padding"],
-                groups=params["groups"],
-                pad_output=params["pad_output"],
+                indices=params["indices"],
+                indexed_axes=params["indexed_axes"],
             )
         return
-    out = conv2D_numpy(
-        x=input,
+    out = blur_at_indices(
+        arr=input,
         kernel=kernel,
-        stride=params["stride"],
-        padding=params["padding"],
-        groups=params["groups"],
-        pad_output=params["pad_output"],
+        indices=params["indices"],
+        indexed_axes=params["indexed_axes"],
     )
     if "shape" in expected:
         assert expected["shape"] == out.shape, "Test failed."
@@ -563,79 +497,62 @@ def test_conv2D_numpy(
             {
                 "patch_size": 4,
                 "coords": 0,
-                "expand_first_dim": False,
             },
-            {"value": (slice(0, 4, None), )},
+            {"value": (np.arange(0, 4),)},
         ),
         (
             {
                 "patch_size": 4,
-                "coords": (0, ),
-                "expand_first_dim": False,
+                "coords": (0,),
             },
-            {"value": (slice(0, 4, None), )},
+            {"value": (np.arange(0, 4),)},
         ),
         (
             {
                 "patch_size": 4,
                 "coords": (0, 0),
-                "expand_first_dim": False,
             },
-            {"value": (slice(0, 4, None), slice(0, 4, None))},
+            {"value": (np.arange(0, 4), np.arange(0, 4))},
         ),
         (
             {
                 "patch_size": (4, 4),
                 "coords": (0, 0),
-                "expand_first_dim": False,
             },
-            {"value": (slice(0, 4, None), slice(0, 4, None))},
+            {"value": (np.arange(0, 4), np.arange(0, 4))},
         ),
         (
             {
                 "patch_size": (4, 6),
                 "coords": (0, 0),
-                "expand_first_dim": False,
             },
-            {"value": (slice(0, 4, None), slice(0, 6, None))},
+            {"value": (np.arange(0, 4), np.arange(0, 6))},
         ),
         (
             {
                 "patch_size": 10,
                 "coords": (1, 2),
-                "expand_first_dim": False,
             },
-            {"value": (slice(1, 11, None), slice(2, 12, None))},
+            {"value": (np.arange(1, 11), np.arange(2, 12))},
         ),
         (
             {
                 "patch_size": (10, 5),
                 "coords": (1, 2),
-                "expand_first_dim": False,
             },
-            {"value": (slice(1, 11, None), slice(2, 7, None))},
-        ),
-        (
-            {
-                "patch_size": 4,
-                "coords": (0, 0),
-                "expand_first_dim": True,
-            },
-            {"value": (slice(None, None, None), slice(0, 4, None), slice(0, 4, None))},
+            {"value": (np.arange(1, 11), np.arange(2, 7))},
         ),
         (
             {
                 "patch_size": 4,
                 "coords": (0, 0, 0),
-                "expand_first_dim": False,
             },
-            {"value": (slice(0, 4, None), slice(0, 4, None), slice(0, 4, None))},
+            {"value": (np.arange(0, 4), np.arange(0, 4), np.arange(0, 4))},
         ),
         (
             {
                 "patch_size": (4, 4, 4),
                 "coords": (0, 0),
-                "expand_first_dim": False,
             },
             {"exception": ValueError},
         ),
@@ -643,23 +560,13 @@ def test_conv2D_numpy(
             {
                 "patch_size": (4, 4),
                 "coords": (0, 0, 0),
-                "expand_first_dim": False,
             },
             {"exception": ValueError},
         ),
         (
             {
                 "patch_size": (4, 4),
-                "coords": (0, ),
-                "expand_first_dim": False,
-            },
-            {"exception": ValueError},
-        ),
-        (
-            {
-                "patch_size": np.ones((4, 4)),
-                "coords": (0, 0),
-                "expand_first_dim": False,
+                "coords": (0,),
             },
             {"exception": ValueError},
         ),
@@ -672,19 +579,11 @@ def test_create_patch_slice(params: dict, expected: Any):
         return
 
     out = create_patch_slice(**params)
-    
-    assert all(out_slice == expected_slice
-               for out_slice, expected_slice in zip(out, expected["value"])
+
+    assert np.all(
+        out_slice == expected_slice
+        for out_slice, expected_slice in zip(out, expected["value"])
     ), f"Slices not equal. {out_slice} != {expected_slice}"
-    assert all(isinstance(out_slice.start, int) or out_slice.start is None
-               for out_slice in out
-    ), f"Not all slice starts are integers/None. {out}"
-    assert all(isinstance(out_slice.stop, int) or out_slice.stop is None
-               for out_slice in out
-    ), f"Not all slice stops are integers/None. {out}"
-    assert all(isinstance(out_slice.step, int) or out_slice.step is None
-               for out_slice in out
-    ), f"Not all slice steps are integers/None. {out}"
 
 
 @pytest.mark.utils
@@ -693,50 +592,78 @@ def test_create_patch_slice(params: dict, expected: Any):
     [
         (
             {
-                "a": np.ones((64, 128)),
-                "x": np.ones((64, 3, 128)),
+                "a_batch": np.ones((64, 128)),
+                "x_batch": np.ones((64, 3, 128)),
             },
             {"value": np.ones((64, 1, 128))},
         ),
         (
             {
-                "a": np.ones((64, 128, 128)),
-                "x": np.ones((64, 3, 128, 128)),
+                "a_batch": np.ones((64, 128, 128)),
+                "x_batch": np.ones((64, 3, 128, 128)),
             },
             {"value": np.ones((64, 1, 128, 128))},
         ),
         (
             {
-                "a": np.ones((64, 1, 128, 128)),
-                "x": np.ones((64, 3, 128, 128)),
+                "a_batch": np.ones((64, 1, 128, 128)),
+                "x_batch": np.ones((64, 3, 128, 128)),
             },
             {"value": np.ones((64, 1, 128, 128))},
         ),
         (
             {
-                "a": np.ones((64, 3, 128, 128)),
-                "x": np.ones((64, 3, 128, 128)),
+                "a_batch": np.ones((64, 3, 128, 128)),
+                "x_batch": np.ones((64, 3, 128, 128)),
             },
             {"value": np.ones((64, 3, 128, 128))},
         ),
         (
             {
-                "a": np.ones((64, 3, 128, 128)),
-                "x": np.ones((32, 3, 128, 128)),
+                "a_batch": np.ones((64, 1, 1, 128)),
+                "x_batch": np.ones((64, 3, 128, 128)),
+            },
+            {"value": np.ones((64, 1, 1, 128))},
+        ),
+        (
+            {
+                "a_batch": np.ones((64, 3, 128)),
+                "x_batch": np.ones((64, 3, 128, 128)),
+            },
+            {"value": np.ones((64, 3, 128, 1))},
+        ),
+        (
+            {
+                "a_batch": np.ones((64, 3)),
+                "x_batch": np.ones((64, 3, 128, 128)),
+            },
+            {"value": np.ones((64, 3, 1, 1))},
+        ),
+        (
+            {
+                "a_batch": np.ones((64, 200)),
+                "x_batch": np.ones((64, 3, 128, 200)),
+            },
+            {"value": np.ones((64, 1, 1, 200))},
+        ),
+        (
+            {
+                "a_batch": np.ones((64, 3, 128, 128)),
+                "x_batch": np.ones((32, 3, 128, 128)),
             },
             {"exception": ValueError},
         ),
         (
             {
-                "a": np.ones((64, 3, 128, 128)),
-                "x": np.ones((64, 3, 128)),
+                "a_batch": np.ones((64, 3, 128, 128)),
+                "x_batch": np.ones((64, 3, 128)),
             },
             {"exception": ValueError},
         ),
         (
             {
-                "a": np.ones((64, 128)),
-                "x": np.ones((64, 3, 128, 128)),
+                "a_batch": np.ones((64, 128)),
+                "x_batch": np.ones((64, 3, 128, 128)),
             },
             {"exception": ValueError},
         ),
@@ -760,21 +687,21 @@ def test_expand_attribution_channel(params: dict, expected: Any):
         (
             {
                 "patch_size": 4,
-                "shape": (16, ),
+                "shape": (16,),
             },
             {"value": 4},
         ),
         (
             {
-                "patch_size": (4, ),
-                "shape": (16, ),
+                "patch_size": (4,),
+                "shape": (16,),
             },
             {"value": 4},
         ),
         (
             {
                 "patch_size": (4, 4),
-                "shape": (16, ),
+                "shape": (16,),
             },
             {"exception": ValueError},
         ),
@@ -808,7 +735,7 @@ def test_expand_attribution_channel(params: dict, expected: Any):
         ),
         (
             {
-                "patch_size": (4, ),
+                "patch_size": (4,),
                 "shape": (16, 16, 16),
             },
             {"value": 64},
@@ -830,3 +757,261 @@ def test_get_nr_patches(params: dict, expected: Any):
 
     out = get_nr_patches(**params)
     assert out == expected["value"]
+
+
+# TODO: Change test cases (and function) for batching update, since currently single images are expected
+@pytest.mark.utils
+@pytest.mark.parametrize(
+    "params,expected",
+    [
+        (
+            {
+                "x_batch": np.ones((30, 2, 3, 4, 5, 6)),
+                "a_batch": np.ones((30, 2, 3, 4, 5, 6)),
+            },
+            {"value": [0, 1, 2, 3, 4]},
+        ),
+        (
+            {
+                "x_batch": np.ones((30, 2, 3, 4, 5, 6)),
+                "a_batch": np.ones((30, 3, 4, 5, 6)),
+            },
+            {"value": [1, 2, 3, 4]},
+        ),
+        (
+            {
+                "x_batch": np.ones((30, 2, 3, 4, 5, 6)),
+                "a_batch": np.ones((30, 4, 5, 6)),
+            },
+            {"value": [2, 3, 4]},
+        ),
+        (
+            {
+                "x_batch": np.ones((30, 2, 3, 4, 5, 6)),
+                "a_batch": np.ones((30, 2, 3, 4)),
+            },
+            {"value": [0, 1, 2]},
+        ),
+        (
+            {
+                "x_batch": np.ones((30, 2, 3, 4, 5, 6)),
+                "a_batch": np.ones((30,)),
+            },
+            {"value": []},
+        ),
+        (
+            {
+                "x_batch": np.ones((30, 2, 3, 4, 5, 6)),
+                "a_batch": np.ones((30, 1, 1, 1, 5, 6)),
+            },
+            {"value": [3, 4]},
+        ),
+        (
+            {
+                "x_batch": np.ones((30, 2, 3, 4, 5, 6)),
+                "a_batch": np.ones((30, 1, 1, 5, 6)),
+            },
+            {"value": [3, 4]},
+        ),
+        (
+            {
+                "x_batch": np.ones((30, 2, 3, 4, 5, 6)),
+                "a_batch": np.ones((30, 2, 3, 1, 1, 1)),
+            },
+            {"value": [0, 1]},
+        ),
+        (
+            {
+                "x_batch": np.ones((30, 2, 3, 4, 5, 6)),
+                "a_batch": np.ones((30, 9, 9, 9, 9, 9, 9)),
+            },
+            {"exception": ValueError},
+        ),
+        (
+            {
+                "x_batch": np.ones((30, 2, 2, 2, 2, 2)),
+                "a_batch": np.ones((30, 2)),
+            },
+            {"exception": ValueError},
+        ),
+        (
+            {
+                "x_batch": np.ones((30, 2, 2)),
+                "a_batch": np.ones((30, 2, 2, 2)),
+            },
+            {"exception": ValueError},
+        ),
+        (
+            {
+                "x_batch": np.ones((30, 2, 3, 4, 5, 6)),
+                "a_batch": np.ones((30, 2, 4, 6)),
+            },
+            {"exception": ValueError},
+        ),
+        (
+            {
+                "x_batch": np.ones((30, 2, 2, 2, 2)),
+                "a_batch": np.ones((30, 2, 1, 1, 2)),
+            },
+            {"exception": ValueError},
+        ),
+    ],
+)
+def test_infer_attribution_axes(params: dict, expected: Any):
+    if "exception" in expected:
+        with pytest.raises(expected["exception"]):
+            out = infer_attribution_axes(**params)
+        return
+
+    out = infer_attribution_axes(**params)
+    assert all([a == b for a, b in list(zip(out, expected["value"]))])
+
+
+# TODO: Change test cases (and function) for batching update, since currently single images are expected
+@pytest.mark.utils
+@pytest.mark.parametrize(
+    "params,expected",
+    [
+        (
+            {
+                "arr": np.ones((2, 3, 4, 5, 6)),
+                "indices": 5,
+                "indexed_axes": [0, 1, 2, 3, 4],
+            },
+            {
+                "value": (
+                    (
+                        np.array([[[[[0]]]]]),
+                        np.array([[[[[0]]]]]),
+                        np.array([[[[[0]]]]]),
+                        np.array([[[[[0]]]]]),
+                        np.array([[[[[5]]]]]),
+                    )
+                )
+            },
+        ),
+        (
+            {
+                "arr": np.ones((2, 3, 4, 5, 6)),
+                "indices": [5, 19],
+                "indexed_axes": [2, 3, 4],
+            },
+            {
+                "value": (
+                    slice(None),
+                    slice(None),
+                    np.array([[[0, 0]]]),
+                    np.array([[[0, 3]]]),
+                    np.array([[[5, 1]]]),
+                )
+            },
+        ),
+        (
+            {
+                "arr": np.ones((2, 3, 4, 5, 6)),
+                "indices": (np.array([1, 1]), np.array([0, 2])),
+                "indexed_axes": [0, 1],
+            },
+            {"value": (np.array([[1, 1]]), np.array([[0, 2]]))},
+        ),
+        (
+            {
+                "arr": np.ones((2,)),
+                "indices": [1],
+                "indexed_axes": [0, 1, 2, 3],
+            },
+            {"exception": AssertionError},
+        ),
+        (
+            {
+                "arr": np.ones((2, 3, 4, 5, 6)),
+                "indices": (np.array([1, 1]), np.array([0, 2])),
+                "indexed_axes": [0],
+            },
+            {"exception": ValueError},
+        ),
+        (
+            {
+                "arr": np.ones((2, 3, 4, 5, 6)),
+                "indices": [1],
+                "indexed_axes": [0, 2],
+            },
+            {"exception": AssertionError},
+        ),
+        (
+            {
+                "arr": np.ones((2, 3, 4, 5, 6)),
+                "indices": [1],
+                "indexed_axes": [2, 3],
+            },
+            {"exception": AssertionError},
+        ),
+    ],
+)
+def test_expand_indices(params: dict, expected: Any):
+    if "exception" in expected:
+        with pytest.raises(expected["exception"]):
+            out = expand_indices(**params)
+        return
+
+    out = expand_indices(**params)
+    assert all([np.all(a == b) for a, b in list(zip(out, expected["value"]))])
+
+
+# TODO: Change test cases (and function) for batching update, since currently single images are expected
+@pytest.mark.utils
+@pytest.mark.parametrize(
+    "params,expected",
+    [
+        (
+            {
+                "arr": np.ones((2, 3, 4, 5, 6)),
+                "axes": [0, 1, 2, 3, 4],
+            },
+            {"value": ()},
+        ),
+        (
+            {
+                "arr": np.ones((2, 3, 4, 5, 6)),
+                "axes": np.array([0, 1]),
+            },
+            {"value": (4, 5, 6)},
+        ),
+        (
+            {
+                "arr": np.ones((2, 3, 4, 5, 6)),
+                "axes": np.array([1, 2, 3, 4]),
+            },
+            {"value": (2,)},
+        ),
+        (
+            {
+                "arr": np.ones((2, 3, 4, 5, 6)),
+                "axes": np.array([0, 2]),
+            },
+            {"exception": AssertionError},
+        ),
+        (
+            {
+                "arr": np.ones((2, 3, 4, 5, 6)),
+                "axes": np.array([1, 2]),
+            },
+            {"exception": AssertionError},
+        ),
+        (
+            {
+                "arr": np.ones((2, 3, 4, 5, 6)),
+                "axes": np.array([0, 1, 2, 3, 4, 5]),
+            },
+            {"exception": AssertionError},
+        ),
+    ],
+)
+def test_get_leftover_shape(params: dict, expected: Any):
+    if "exception" in expected:
+        with pytest.raises(expected["exception"]):
+            out = get_leftover_shape(**params)
+        return
+
+    out = get_leftover_shape(**params)
+    assert all([np.all(a == b) for a, b in list(zip(out, expected["value"]))])
