@@ -32,6 +32,24 @@ from ...quantus.helpers.explanation_func import explain
             },
             {"min": -1.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_func": baseline_replacement_by_indices,
+                "nr_runs": 10,
+                "perturb_baseline": "mean",
+                "similarity_func": correlation_spearman,
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "max_steps_per_input": 2,
+                "disable_warnings": False,
+                "display_progressbar": False,
+                "softmax": True,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
         (
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
@@ -224,6 +242,24 @@ def test_faithfulness_correlation(
                 "max_steps_per_input": 2,
                 "disable_warnings": False,
                 "display_progressbar": False,
+                "softmax": False,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_func": baseline_replacement_by_indices,
+                "features_in_step": 196,
+                "perturb_baseline": "uniform",
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "max_steps_per_input": 2,
+                "disable_warnings": False,
+                "display_progressbar": False,
+                "softmax": True,
             },
             {"min": -1.0, "max": 1.0},
         ),
@@ -239,6 +275,7 @@ def test_faithfulness_correlation(
                 "method": "Gradient",
                 "disable_warnings": True,
                 "display_progressbar": False,
+                "softmax": False,
             },
             {"min": -1.0, "max": 1.0},
         ),
@@ -257,6 +294,7 @@ def test_faithfulness_correlation(
                 "disable_warnings": True,
                 "display_progressbar": False,
                 "a_batch_generate": False,
+                "softmax": False,
             },
             {"min": 0.0, "max": 1.0},
         ),
@@ -273,6 +311,7 @@ def test_faithfulness_correlation(
                 "max_steps_per_input": 2,
                 "disable_warnings": True,
                 "display_progressbar": True,
+                "softmax": False,
             },
             {"min": -1.0, "max": 1.0},
         ),
@@ -288,6 +327,7 @@ def test_faithfulness_correlation(
                 "disable_warnings": False,
                 "display_progressbar": False,
                 "a_batch_generate": False,
+                "softmax": False,
             },
             {"min": -1.0, "max": 1.0},
         ),
@@ -345,7 +385,7 @@ def test_faithfulness_estimate(
                 "disable_warnings": False,
                 "display_progressbar": False,
             },
-            {"min": 0.0, "max": 1.0},
+            {"min": 0.0, "max": 80.0},
         ),
         (
             lazy_fixture("load_mnist_model"),
@@ -361,7 +401,7 @@ def test_faithfulness_estimate(
                 "display_progressbar": False,
                 "a_batch_generate": False,
             },
-            {"min": 0.0, "max": 1.0},
+            {"min": 0.0, "max": 80.0},
         ),
         (
             lazy_fixture("load_mnist_model"),
@@ -376,7 +416,7 @@ def test_faithfulness_estimate(
                 "disable_warnings": True,
                 "display_progressbar": True,
             },
-            {"min": 0.0, "max": 1.0},
+            {"min": 0.0, "max": 80.0},
         ),
         (
             lazy_fixture("load_1d_3ch_conv_model"),
@@ -437,7 +477,9 @@ def test_iterative_removal_of_features(
         **params,
     )
 
-    assert scores is not None, "Test failed."
+    assert all(
+        ((s >= expected["min"]) & (s <= expected["max"])) for s in scores
+    ), "Test failed."
 
 
 @pytest.mark.faithfulness
@@ -672,6 +714,7 @@ def test_monotonicity_nguyen(
                 "disable_warnings": False,
                 "display_progressbar": False,
                 "a_batch_generate": True,
+                "return_auc": False,
             },
             {"min": 0.0, "max": 1.0},
         ),
@@ -687,6 +730,7 @@ def test_monotonicity_nguyen(
                 "disable_warnings": True,
                 "display_progressbar": False,
                 "a_batch_generate": True,
+                "return_auc": False,
             },
             {"min": 0.0, "max": 1.0},
         ),
@@ -694,7 +738,7 @@ def test_monotonicity_nguyen(
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
             {
-                "perturb_baseline": "random",
+                "perturb_baseline": "uniform",
                 "features_in_step": 56,
                 "normalise": False,
                 "explain_func": explain,
@@ -702,6 +746,7 @@ def test_monotonicity_nguyen(
                 "a_batch_generate": True,
                 "disable_warnings": True,
                 "display_progressbar": False,
+                "return_auc": False,
             },
             {"min": 0.0, "max": 1.0},
         ),
@@ -709,7 +754,7 @@ def test_monotonicity_nguyen(
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
             {
-                "perturb_baseline": "random",
+                "perturb_baseline": "uniform",
                 "features_in_step": 112,
                 "normalise": False,
                 "explain_func": explain,
@@ -717,6 +762,7 @@ def test_monotonicity_nguyen(
                 "disable_warnings": True,
                 "display_progressbar": False,
                 "a_batch_generate": False,
+                "return_auc": False,
             },
             {"min": 0.0, "max": 1.0},
         ),
@@ -733,6 +779,7 @@ def test_monotonicity_nguyen(
                 "max_steps_per_input": 2,
                 "disable_warnings": True,
                 "display_progressbar": True,
+                "return_auc": False,
             },
             {"min": 0.0, "max": 1.0},
         ),
@@ -746,9 +793,41 @@ def test_monotonicity_nguyen(
                 "perturb_baseline": "mean",
                 "disable_warnings": True,
                 "a_batch_generate": False,
+                "return_auc": False,
             },
             {"min": 0.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_baseline": "uniform",
+                "features_in_step": 56,
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "abs": True,
+                "max_steps_per_input": 2,
+                "disable_warnings": True,
+                "display_progressbar": True,
+                "return_auc": True,
+            },
+            {"min": 0.0, "max": 14.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d"),
+            {
+                "features_in_step": 10,
+                "normalise": False,
+                "perturb_func": perturb_func.baseline_replacement_by_indices,
+                "perturb_baseline": "mean",
+                "disable_warnings": True,
+                "a_batch_generate": False,
+                "return_auc": True,
+            },
+            {"min": 0.0, "max": 10.0},
+        ),
     ],
 )
 def test_pixel_flipping(
@@ -773,7 +852,10 @@ def test_pixel_flipping(
         a_batch = data["a_batch"]
     else:
         a_batch = None
-    scores = PixelFlipping(**params)(
+
+    metric = PixelFlipping(**params)
+
+    scores = metric(
         model=model,
         x_batch=x_batch,
         y_batch=y_batch,
@@ -781,9 +863,22 @@ def test_pixel_flipping(
         **params,
     )
 
-    assert all(
-        ((s >= expected["min"]) & (s <= expected["max"])) for s in scores[0]
-    ), "Test failed."
+    if params.get("return_auc", True):
+        assert all(
+            [
+                (s >= expected["min"] and s <= expected["max"])
+                for s_list in metric.get_auc_score
+                for s in s_list
+            ]
+        ), "Test failed."
+    else:
+        assert all(
+            [
+                (s >= expected["min"] and s <= expected["max"])
+                for s_list in scores
+                for s in s_list
+            ]
+        ), "Test failed."
 
 
 @pytest.mark.faithfulness
@@ -803,8 +898,9 @@ def test_pixel_flipping(
                 "disable_warnings": False,
                 "display_progressbar": False,
                 "a_batch_generate": True,
+                "return_auc": False,
             },
-            {"min": -1, "max": 1.0},
+            {"min": -1.0, "max": 1.0},
         ),
         (
             lazy_fixture("load_mnist_model"),
@@ -819,8 +915,9 @@ def test_pixel_flipping(
                 "disable_warnings": True,
                 "display_progressbar": False,
                 "a_batch_generate": False,
+                "return_auc": False,
             },
-            {"min": -1, "max": 1.0},
+            {"min": -1.0, "max": 1.0},
         ),
         (
             lazy_fixture("load_mnist_model"),
@@ -833,10 +930,11 @@ def test_pixel_flipping(
                 "disable_warnings": True,
                 "explain_func": explain,
                 "method": "Saliency",
-                "perturb_func": perturb_func.baseline_replacement_by_patch,
+                "perturb_func": perturb_func.baseline_replacement_by_indices,
                 "a_batch_generate": False,
+                "return_auc": False,
             },
-            {"min": 0.0, "max": 1.0},
+            {"min": -1.0, "max": 1.0},
         ),
         (
             lazy_fixture("load_1d_3ch_conv_model"),
@@ -845,8 +943,9 @@ def test_pixel_flipping(
                 "disable_warnings": True,
                 "display_progressbar": False,
                 "a_batch_generate": False,
+                "return_auc": False,
             },
-            {"min": -1, "max": 1.0},
+            {"min": -1.0, "max": 1.0},
         ),
         (
             lazy_fixture("load_mnist_model"),
@@ -860,8 +959,25 @@ def test_pixel_flipping(
                 "method": "Saliency",
                 "disable_warnings": True,
                 "display_progressbar": True,
+                "return_auc": False,
             },
-            {"min": -1, "max": 1.0},
+            {"min": -1.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_baseline": "mean",
+                "patch_size": 7,
+                "normalise": True,
+                "order": "morf",
+                "explain_func": explain,
+                "method": "Saliency",
+                "disable_warnings": True,
+                "display_progressbar": True,
+                "return_auc": True,
+            },
+            {"min": 0.0, "max": 100.0},
         ),
     ],
 )
@@ -887,7 +1003,10 @@ def test_region_perturbation(
         a_batch = data["a_batch"]
     else:
         a_batch = None
-    scores = RegionPerturbation(**params)(
+
+    metric = RegionPerturbation(**params)
+
+    scores = metric(
         model=model,
         x_batch=x_batch,
         y_batch=y_batch,
@@ -895,7 +1014,19 @@ def test_region_perturbation(
         **params,
     )
 
-    assert scores is not None, "Test failed."
+    if params.get("return_auc", True):
+        assert all(
+            ((s >= expected["min"]) & (s <= expected["max"]))
+            for s in metric.get_auc_score
+        ), "Test failed."
+    else:
+        assert all(
+            [
+                (s >= expected["min"] and s <= expected["max"])
+                for _, s_list in scores.items()
+                for s in s_list
+            ]
+        ), "Test failed."
 
 
 @pytest.mark.faithfulness
@@ -915,6 +1046,7 @@ def test_region_perturbation(
                 "max_steps_per_input": 2,
                 "disable_warnings": False,
                 "display_progressbar": False,
+                "return_auc": False,
             },
             {"min": 0.0, "max": 1.0},
         ),
@@ -922,7 +1054,7 @@ def test_region_perturbation(
             lazy_fixture("load_mnist_model"),
             lazy_fixture("load_mnist_images"),
             {
-                "perturb_baseline": "random",
+                "perturb_baseline": "uniform",
                 "patch_size": 4,
                 "normalise": True,
                 "explain_func": explain,
@@ -930,6 +1062,7 @@ def test_region_perturbation(
                 "disable_warnings": True,
                 "display_progressbar": False,
                 "a_batch_generate": False,
+                "return_auc": False,
             },
             {"min": 0.0, "max": 1.0},
         ),
@@ -937,13 +1070,14 @@ def test_region_perturbation(
             lazy_fixture("load_mnist_model_tf"),
             lazy_fixture("load_mnist_images_tf"),
             {
-                "perturb_baseline": "random",
+                "perturb_baseline": "uniform",
                 "patch_size": 4,
                 "normalise": True,
                 "explain_func": explain,
                 "method": "Gradient",
                 "disable_warnings": True,
                 "display_progressbar": False,
+                "return_auc": False,
             },
             {"min": 0.0, "max": 1.0},
         ),
@@ -960,6 +1094,7 @@ def test_region_perturbation(
                 "max_steps_per_input": 2,
                 "disable_warnings": True,
                 "display_progressbar": True,
+                "return_auc": False,
             },
             {"min": 0.0, "max": 1.0},
         ),
@@ -975,9 +1110,43 @@ def test_region_perturbation(
                 "disable_warnings": False,
                 "display_progressbar": False,
                 "a_batch_generate": False,
+                "return_auc": False,
             },
             {"min": 0.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_baseline": "mean",
+                "patch_size": 7,
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "abs": True,
+                "max_steps_per_input": 2,
+                "disable_warnings": False,
+                "display_progressbar": True,
+                "return_auc": True,
+            },
+            {"min": 0.0, "max": 100.0},
+        ),
+        (
+            lazy_fixture("load_1d_3ch_conv_model"),
+            lazy_fixture("almost_uniform_1d"),
+            {
+                "perturb_baseline": "mean",
+                "patch_size": 7,
+                "normalise": True,
+                "abs": True,
+                "max_steps_per_input": 2,
+                "disable_warnings": False,
+                "display_progressbar": False,
+                "a_batch_generate": False,
+                "return_auc": True,
+            },
+            {"min": 0.0, "max": 16.0},
+        ),
     ],
 )
 def test_selectivity(
@@ -1002,7 +1171,10 @@ def test_selectivity(
         a_batch = data["a_batch"]
     else:
         a_batch = None
-    scores = Selectivity(**params)(
+
+    metric = Selectivity(**params)
+
+    scores = metric(
         model=model,
         x_batch=x_batch,
         y_batch=y_batch,
@@ -1010,9 +1182,19 @@ def test_selectivity(
         **params,
     )
 
-    assert all(
-        ((s >= expected["min"]) & (s <= expected["max"])) for s in scores[0]
-    ), "Test failed."
+    if params.get("return_auc", True):
+        assert all(
+            ((s >= expected["min"]) & (s <= expected["max"]))
+            for s in metric.get_auc_score
+        ), "Test failed."
+    else:
+        assert all(
+            [
+                (s >= expected["min"] and s <= expected["max"])
+                for _, s_list in scores.items()
+                for s in s_list
+            ]
+        ), "Test failed."
 
 
 @pytest.mark.faithfulness
@@ -1103,6 +1285,24 @@ def test_selectivity(
             },
             {"min": -1.0, "max": 1.0},
         ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_baseline": "black",
+                "n_max_percentage": 0.9,
+                "features_in_step": 28,
+                "similarity_func": correlation_spearman,
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "abs": True,
+                "disable_warnings": False,
+                "display_progressbar": True,
+                "return_aggregate": True,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
     ],
 )
 def test_sensitivity_n(
@@ -1139,3 +1339,158 @@ def test_sensitivity_n(
     assert all(
         ((s >= expected["min"]) & (s <= expected["max"])) for s in scores
     ), "Test failed."
+    
+
+
+@pytest.mark.faithfulness
+@pytest.mark.parametrize(
+    "model,data,params,expected",
+    [
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_func": baseline_replacement_by_indices,
+                "aggregate": False,
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "abs": True,
+                "disable_warnings": False,
+                "display_progressbar": False,
+                "features_in_step": 8,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_func": baseline_replacement_by_indices,
+                "aggregate": True,
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "abs": True,
+                "disable_warnings": False,
+                "display_progressbar": False,
+                "features_in_step": 8,
+                "a_batch_generate": False,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
+    ],
+)
+def test_infidelity(
+    model,
+    data: np.ndarray,
+    params: dict,
+    expected: Union[float, dict, bool],
+):
+    x_batch, y_batch = (
+        data["x_batch"],
+        data["y_batch"],
+    )
+
+    if params.get("a_batch_generate", True):
+        explain = params["explain_func"]
+        a_batch = explain(
+            model=model,
+            inputs=x_batch,
+            targets=y_batch,
+            **params,
+        )
+    elif "a_batch" in data:
+        a_batch = data["a_batch"]
+    else:
+        a_batch = None
+    score = Infidelity(**params)(
+        model=model,
+        x_batch=x_batch,
+        y_batch=y_batch,
+        a_batch=a_batch,
+        **params,
+    )
+
+    assert score is not None, "Test failed."
+
+
+@pytest.mark.faithfulness
+@pytest.mark.parametrize(
+    "model,data,params,expected",
+    [
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_func": noisy_linear_imputation,
+                "similarity_func": correlation_spearman,
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "abs": True,
+                "disable_warnings": False,
+                "display_progressbar": False,
+                "percentages": list(range(1, 100, 2)),
+                "img_size": 28 * 28,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
+        (
+            lazy_fixture("load_mnist_model"),
+            lazy_fixture("load_mnist_images"),
+            {
+                "perturb_func": noisy_linear_imputation,
+                "similarity_func": correlation_spearman,
+                "normalise": True,
+                "explain_func": explain,
+                "method": "Saliency",
+                "abs": True,
+                "disable_warnings": False,
+                "display_progressbar": False,
+                "percentages": list(range(1, 100, 2)),
+                "a_batch_generate": False,
+                "img_size": 28 * 28,
+            },
+            {"min": -1.0, "max": 1.0},
+        ),
+    ],
+)
+def test_ROAD(
+    model,
+    data: np.ndarray,
+    params: dict,
+    expected: Union[float, dict, bool],
+):
+    x_batch, y_batch = (
+        data["x_batch"],
+        data["y_batch"],
+    )
+
+    if params.get("a_batch_generate", True):
+        explain = params["explain_func"]
+        a_batch = explain(
+            model=model,
+            inputs=x_batch,
+            targets=y_batch,
+            **params,
+        )
+    elif "a_batch" in data:
+        a_batch = data["a_batch"]
+    else:
+        a_batch = None
+    scores = ROAD(**params)(
+        model=model,
+        x_batch=x_batch,
+        y_batch=y_batch,
+        a_batch=a_batch,
+        **params,
+    )
+
+    max_ind = max(scores)
+    min_ind = min(scores)
+
+    assert (scores[min_ind] <= expected["max"]) & (
+        scores[max_ind] >= expected["min"]
+    ), "Test failed."
+
